/////////////////////////////////////////////////////////////////////////////
// Name:        ButtonPropDlg.cpp
// Purpose:     DVD menu button properties dialog
// Author:      Alex Thuering
// Created:		20.11.2003
// RCS-ID:      $Id: ButtonPropDlg.cpp,v 1.19 2005/05/17 11:41:34 ntalex Exp $
// Copyright:  (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////
 
#include "ButtonPropDlg.h"
#include <wx/statline.h>
#include <wx/tokenzr.h>

enum
{
  JUMP_ACTION_RADIO_ID = 7900,
  CUSTOM_ACTION_RADIO_ID,
  TSI_CHOICE_ID,
  PGCI_CHOICE_ID,
};

BEGIN_EVENT_TABLE(ButtonPropDlg, PropDlg)
  EVT_RADIOBUTTON(JUMP_ACTION_RADIO_ID, ButtonPropDlg::OnJumpActionSelected)
  EVT_RADIOBUTTON(CUSTOM_ACTION_RADIO_ID, ButtonPropDlg::OnCustomActionSelected)
  EVT_CHOICE(TSI_CHOICE_ID, ButtonPropDlg::OnChangeTitleset)
  EVT_CHOICE(PGCI_CHOICE_ID, ButtonPropDlg::OnChangeTarget)
END_EVENT_TABLE()

ButtonPropDlg::ButtonPropDlg(wxWindow* parent,
  MenuButton* object, Menu *menu, DVD* dvd, int tsi, int pgci):
	PropDlg(parent), m_object(object), m_menu(menu), m_dvd(dvd),
	m_tsi(tsi), m_pgci(pgci)
{
  Create();
}

void ButtonPropDlg::CreatePropPanel(wxSizer* sizer)
{
  wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);
  wxFlexGridSizer* grid;
  
  AddStaticLine(mainSizer, _("Action"));
  grid = new wxFlexGridSizer(2, 2, 4, 16);
  grid->AddGrowableCol(1);
  AddRadioProp(grid, _("Jump to"), !m_object->HasCustomAction(), wxRB_GROUP,
	false, JUMP_ACTION_RADIO_ID);
  wxBoxSizer* jumpSizer = new wxBoxSizer(wxHORIZONTAL);
  AddChoiceProp(jumpSizer, _(""), _(""), wxArrayString(), TSI_CHOICE_ID);
  m_titlesetChoice = (wxChoice*) m_controls[m_controls.Count()-1];
  UpdateTitlesets();
  AddChoiceProp(jumpSizer, _(""), _(""), wxArrayString(), PGCI_CHOICE_ID);
  m_targetChoice = (wxChoice*) m_controls[m_controls.Count()-1];
  UpdateTargets();
  AddChoiceProp(jumpSizer, _(""), _(""), wxArrayString());
  m_chapterChoice = (wxChoice*) m_controls[m_controls.Count()-1];
  UpdateChapters();
  grid->Add(jumpSizer);
  
  wxString customAction = m_object->GetCustomAction();
  if (!customAction.length())
  {
	// check if selection was ok, otherwise set this action as custom action
	int tsi = m_object->GetActionTsi() > -2 ? m_object->GetActionTsi() : m_tsi;
	if (GetTsi() != tsi || GetPgci() != m_object->GetActionPgci() ||
		GetChapter() != m_object->GetActionChapter())
	  customAction = m_object->GetAction();
  }
  AddRadioProp(grid, _("Custom"), customAction.length(), 0,
	false, CUSTOM_ACTION_RADIO_ID);
  AddTextProp(grid, _(""), customAction);
  m_customActionEdit = (wxTextCtrl*) m_controls[m_controls.Count()-1];
  mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 8);
  
  wxCommandEvent evt;
  if (customAction.length())
  {
	OnCustomActionSelected(evt);
  }
  else
	OnJumpActionSelected(evt);
  
  //AddStaticLine(mainSizer, _("Focus"));
  grid = new wxFlexGridSizer(3, 4, 4, 16);
  grid->AddGrowableCol(0);
  grid->Add(10,10); AddText(grid, _("Focus")); AddDirectionProp(grid, bdUP);   grid->Add(10,10);
  grid->Add(10,10); AddDirectionProp(grid, bdLEFT); grid->Add(10,10); AddDirectionProp(grid, bdRIGHT);
  grid->Add(10,10); grid->Add(10,10); AddDirectionProp(grid, bdDOWN); grid->Add(10,10);
  mainSizer->Add(grid, 0, wxEXPAND|wxALL, 10);
  
  AddStaticLine(mainSizer, _("Button look"));
  wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
  AddTextProp(sizer2, _("Label: "), m_object->GetParam(_T("text")));
  AddFontProp(sizer2, _T(""), m_object->GetParamFont(_T("text_font")), _("Font..."));
  mainSizer->Add(sizer2, 0, wxEXPAND|wxTOP|wxBOTTOM, 8);
  
  //AddStaticLine(mainSizer, _("Colours"));
  grid = new wxFlexGridSizer(3, 4, 4, 16);
  AddText(grid, _T(""));
  AddText(grid, _("Normal"));
  AddText(grid, _("Highlighted"));
  AddText(grid, _("Selected"));
  AddColourProp(grid, _("Text colour:"), m_object->GetParamColour(_T("text_colour")));
  AddColourProp(grid, _T(""), m_object->GetParamColour(_T("text_colour"), mbsHIGHLIGHTED));
  AddColourProp(grid, _T(""), m_object->GetParamColour(_T("text_colour"), mbsSELECTED));
  AddColourProp(grid, _("BG colour:"), m_object->GetBackgroundColour());
  AddColourProp(grid, _T(""), m_object->GetBackgroundColour(mbsHIGHLIGHTED));
  AddColourProp(grid, _T(""), m_object->GetBackgroundColour(mbsSELECTED));
  mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 8);
  
  mainSizer->Add(10, 10, 1, wxEXPAND);
  sizer->Add(mainSizer, 1, wxEXPAND|wxALL, 10);
}

int ButtonPropDlg::GetTsi()
{
  if (m_titlesetChoice->GetSelection() >= 0)
	return ((int) m_titlesetChoice->GetClientData(m_titlesetChoice->GetSelection()))-1;
  return 0;
}

int ButtonPropDlg::GetPgci()
{
  if (m_targetChoice->GetSelection() >= 0)
	return (int) m_targetChoice->GetClientData(m_targetChoice->GetSelection());
  return 0;
}

int ButtonPropDlg::GetChapter()
{
  if (m_chapterChoice->GetSelection() >= 0)
	return m_chapterChoice->GetSelection();
  return 0;
}

void ButtonPropDlg::UpdateTitlesets()
{
  int sel = 0;
  wxChoice* ctrl = m_titlesetChoice;
  int tsi = m_object->GetActionTsi() > -2 ? m_object->GetActionTsi() : m_tsi;
  ctrl->Clear();
  if (m_dvd->GetVmgm().GetCount()>0)
	ctrl->Append(_("video manager"), (void*) 0);
  for (int i=0; i<(int)m_dvd->GetTitlesets().Count(); i++)
  {
	if (!m_dvd->IsJumppadsEnabled() && m_tsi >= 0 && i != m_tsi)
	  continue;
	ctrl->Append(wxString::Format(_("titleset %d"), i+1), (void*) (i+1));
	if (i == tsi)
	  sel = ctrl->GetCount()-1; 
  }
  ctrl->SetSelection(sel);
  ctrl->Enable(ctrl->GetCount()>1);
}

void ButtonPropDlg::UpdateTargets()
{
  int sel = 0;
  wxChoice* ctrl = m_targetChoice;
  int tsi = GetTsi();
  int pgci = ctrl->GetCount() ? GetPgci() : m_object->GetActionPgci();
  ctrl->Clear();
  PgcArray& pgcs = m_dvd->GetPgcArray(tsi, true);
  for (int i=0; i<(int)pgcs.Count(); i++)
  {
	if (!m_dvd->IsJumppadsEnabled() && m_tsi==-1 && tsi>=0 && i>0)
	  continue;
	ctrl->Append(wxString::Format(_("menu %d"), i+1), (void*) (i*2));
	if (i*2 == pgci)
	  sel = ctrl->GetCount()-1;
  }
  if (tsi>=0 && (m_dvd->IsJumppadsEnabled() || m_tsi>=0))
  {
	PgcArray& pgcs = m_dvd->GetPgcArray(tsi, false);
	for (int i=0; i<(int)pgcs.Count(); i++)
	{
	  ctrl->Append(wxString::Format(_("title %d"), i+1), (void*) (i*2+1));
	  if (i*2+1 == pgci)
		sel = ctrl->GetCount()-1;
	}
  }
  ctrl->SetSelection(sel);
}

void ButtonPropDlg::UpdateChapters()
{
  wxChoice* ctrl = m_chapterChoice;
  int tsi = GetTsi();
  int pgci = GetPgci();
  int chapter = ctrl->GetCount() ? GetChapter() : m_object->GetActionChapter();
  ctrl->Clear();
  ctrl->Append(wxString::Format(_("chapter %d"), 1));
  if (tsi>=0 && pgci%2 == 1)
  {
	PgcArray& pgcs = m_dvd->GetPgcArray(tsi, false);
	if (pgci/2 < (int)pgcs.Count())
	{
	  Pgc& pgc = *pgcs[pgci/2];
	  for (int i=0; i<(int)pgc.GetVobs().Count(); i++)
	  {
		Vob& vob = *pgc.GetVobs()[i];
		wxStringTokenizer tkz(vob.GetChapters(), _T(","));
		while (tkz.HasMoreTokens())
		{
		  wxString token = tkz.GetNextToken();
		  if (token != _T("0"))
			ctrl->Append(wxString::Format(_("chapter %d"), ctrl->GetCount()+1));
		}
	  }
	}
  }
  ctrl->SetSelection(chapter);
  ctrl->Enable(ctrl->GetCount()>1);
}

void ButtonPropDlg::OnJumpActionSelected(wxCommandEvent& WXUNUSED(evt))
{
  m_titlesetChoice->Enable(m_titlesetChoice->GetCount()>1);
  m_targetChoice->Enable(true);
  m_chapterChoice->Enable(m_chapterChoice->GetCount()>1);
  m_customActionEdit->Enable(false);
}

void ButtonPropDlg::OnCustomActionSelected(wxCommandEvent& WXUNUSED(evt))
{
  m_titlesetChoice->Enable(false);
  m_targetChoice->Enable(false);
  m_chapterChoice->Enable(false);
  m_customActionEdit->Enable(true);
  if (m_customActionEdit->GetValue().length() == 0)
  {
	MenuButton* button = new MenuButton();
	button->SetActionTsi(GetTsi() != m_tsi ? GetTsi() : -2);
	button->SetActionPgci(GetPgci());
	button->SetActionChapter(GetChapter());
	m_customActionEdit->SetValue(button->GetAction());
  }
}

void ButtonPropDlg::AddStaticLine(wxSizer* sizer, wxString caption)
{
  wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
  wxStaticText* staticText = new wxStaticText(this, -1, caption);
  staticText->SetFont(wxFont(12, wxDEFAULT, wxNORMAL, wxBOLD, 0, _T("")));
  sizer2->Add(staticText, 0, wxRIGHT|wxTOP|wxBOTTOM, 5);
  sizer2->Add(new wxStaticLine(this, -1), 1, wxALIGN_CENTER_VERTICAL, 5);
  sizer->Add(sizer2, 0, wxEXPAND, 0);
}

void ButtonPropDlg::AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir)
{
	wxArrayString buttons;
  	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 1, 4, 16);
	
	// add all buttons except current
	buttons.Add(_("auto"));
	int btInd = 1;
	for ( int i=0; i<m_menu->GetObjectsCount(); i++)
	{
		MenuObject* obj = m_menu->GetObject(i);
		if ( obj->GetType() == moBUTTON)
		{
			if ( obj != m_object)
			{
				wxString text = ((MenuButton*) obj)->GetParam(_T("text"));
				if (!text.length())
				  text = wxString::Format(_T("button%d"), btInd);
				buttons.Add(text);
			}
			else
			  m_btInd = btInd-1;
			btInd++;
		}
	}
	int defValue = m_object->GetDirection(dir-bdLEFT)+1;
	if (defValue>m_btInd)
	  defValue--;
  	AddChoiceProp(grid, _T(""), buttons[defValue], buttons, wxCB_READONLY);
	wxChoice* ctrl = (wxChoice*) m_controls[m_controls.Count()-1];
	ctrl->SetSizeHints(ctrl->GetSize().GetWidth()*3/2, -1, ctrl->GetSize().GetWidth()*3/2, -1);
	sizer->Add(grid, 0, 0, 0);
}

bool ButtonPropDlg::SetValues()
{
  if (!m_dvd->IsJumppadsEnabled() && m_tsi<0 && GetString(0).Find(_T("chapter"))>=0)
  {
	wxMessageBox(_("You cannot jump from vmgMenu to a chapter."),
	  GetTitle(), wxOK|wxICON_ERROR, this);
	return false;
  }
  if (GetBool(4) && GetString(5).length() == 0) // empty custom action
  {
	wxMessageBox(_("Please enter the action command."),
	  GetTitle(), wxOK|wxICON_ERROR, this);
	return false;
  }
  int n = 1;
  int tsi = ((int)GetClientData(n++))-1;
  m_object->SetActionTsi(tsi != m_tsi ? tsi : -2);
  m_object->SetActionPgci((int)GetClientData(n++));
  m_object->SetActionChapter(GetInt(n++));
  if (GetBool(n++))
	m_object->SetCustomAction(GetString(n));
  else
	m_object->SetCustomAction(_T(""));
  n++;
  
  int btInd = GetInt(n++)-1; if (btInd >= m_btInd) btInd++;
  m_object->SetDirection(2, btInd);
  btInd = GetInt(n++)-1; if (btInd >= m_btInd) btInd++;
  m_object->SetDirection(0, btInd);
  btInd = GetInt(n++)-1; if (btInd >= m_btInd) btInd++;
  m_object->SetDirection(1, btInd);
  btInd = GetInt(n++)-1; if (btInd >= m_btInd) btInd++;
  m_object->SetDirection(3, btInd);
  
  m_object->SetParam(_T("text"), GetString(n++));
  m_object->SetParamFont(_T("text_font"), GetFont(n++).GetChosenFont());
  
  m_object->SetParamColour(_T("text_colour"), GetColour(n++));
  m_object->SetParamColour(_T("text_colour"), GetColour(n++), mbsHIGHLIGHTED);
  m_object->SetParamColour(_T("text_colour"), GetColour(n++), mbsSELECTED);
  m_object->SetBackgroundColour(GetColour(n++));
  m_object->SetBackgroundColour(GetColour(n++), mbsHIGHLIGHTED);
  m_object->SetBackgroundColour(GetColour(n++), mbsSELECTED);
  return true;
}

