/////////////////////////////////////////////////////////////////////////////
// Name:        DVD.cpp
// Purpose:     The class to store a DVD Structure (Titles/Menus)
// Author:      Alex Thuering
// Created:		29.01.2003
// RCS-ID:      $Id: DVD.cpp,v 1.19 2005/04/15 09:20:33 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "DVD.h"
#include "Config.h"
#include <wxXML/xmlhelpr.h>
#include <sys/stat.h> 

////////////////////////////////// Vob //////////////////////////////////////

wxXmlNode* Vob::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("vob"));
  node->AddProperty(_T("file"),
    type == DVDAUTHOR_XML && GetTmpFilename().length() ?
	GetTmpFilename() : GetFilename());
  if (type == DVDSTYLER_XML)
  {
	if (GetVideoFilename().length())
	  XmlWriteValue(node, _T("video"), GetVideoFilename());
	if (GetAudioFilename().length())
	  XmlWriteValue(node, _T("audio"), GetAudioFilename());
  }
	  
  if (GetPause() != 0)
  {
	wxString pause = _T("inf");
	if (GetPause()>0)
	  pause = wxString::Format(_T("%d"), GetPause());
	node->AddProperty(_T("pause"), pause);
  }
  if (GetChapters().length())
	node->AddProperty(_T("chapters"), GetChapters());
  return node;
}

bool Vob::PutXML(wxXmlNode* node)
{
  wxString val;
  long lval;
  
  if (node->GetPropVal(_T("file"), &val))
	m_filename = val;
  wxXmlNode* child = node->GetChildren();
  while (child)
  {
	if (child->GetName() == _T("video"))
	  SetVideoFilename(child->GetChildren()->GetContent());
	if (child->GetName() == _T("audio"))
	  SetAudioFilename(child->GetChildren()->GetContent());
	child = child->GetNext();
  }
  if (node->GetPropVal(_T("pause"), &val) && val.ToLong(&lval))
	m_pause = int(lval);
  if (node->GetPropVal(_T("chapters"), &val))
	m_chapters = val;
  return true;
}

unsigned int Vob::GetFileSize(wxString filename)
{
  wxStructStat fInfo;
  if (wxStat(filename, &fInfo) != 0)
	return 0;
  return fInfo.st_size/1024;
}

int Vob::GetSize(bool generated)
{
  if (generated)
  {
	if (GetTmpFilename().length())
	  return GetFileSize(GetTmpFilename());
	else if (GetFilename().length())
	  return GetFileSize(GetFilename());
	return 0;
  }
  
  int size = 0;
  if (GetFilename().length())
	size += GetFileSize(GetFilename());
  if (GetVideoFilename().length())
	size += GetFileSize(GetVideoFilename());
  if (GetAudioFilename().length())
	size += GetFileSize(GetAudioFilename());
  if (GetMenu())
	size += 250;
  return size;
}

////////////////////////////////// Pgc //////////////////////////////////////

Menu* Pgc::GetMenu()
{
  for (int i=0; i<(int)GetVobs().GetCount(); i++)
	if (GetVobs()[i]->GetMenu())
	  return GetVobs()[i]->GetMenu();
  return NULL;
}

wxXmlNode* Pgc::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("pgc"));
  if (m_entry.length())
	node->AddProperty(_T("entry"), m_entry);
  
  Menu* menu = GetMenu();
  if (menu)
  {
	if (type == DVDSTYLER_XML) // DVDSTYLER_XML
	{
	  wxXmlNode* spuNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("spumux"));
	  spuNode->AddChild(menu->GetXML(type));
	  node->AddChild(spuNode);
	}
	else if (type == DVDAUTHOR_XML) // DVDAUTHOR_XML
	  menu->GetXML(type, node);
  }
  
  // DVDAUTHOR_XML & DVDSTYLER_XML
  for (int i=0; i<(int)GetVobs().GetCount(); i++)
	node->AddChild(GetVobs()[i]->GetXML(type));
  if (GetPreCommands().length())
	XmlWriteValue(node, _T("pre"), GetPreCommands());
  if (GetPostCommands().length())
	XmlWriteValue(node, _T("post"), GetPostCommands());
  
  return node;
}

bool Pgc::PutXML(wxXmlNode* node)
{
  WX_CLEAR_ARRAY(GetVobs());
  
  wxString val;
  if (node->GetPropVal(_T("entry"), &val))
	m_entry = val;
  
  wxXmlNode* child = node->GetChildren();
  Menu* menu = NULL;
	
  while (child)
  {
	  // get spunode
	  if (child->GetName() == _T("spumux"))
	  {
		  // get menu
		  menu = new Menu();
		  menu->PutXML(child);
	  }
	  // get dvdauth nodes
	  else if (child->GetName() == _T("vob"))
	  {
		  Vob* vob;
		  if (menu)
			  vob = new Vob(menu);
		  else
			  vob = new Vob(_T(""));
		  if (vob->PutXML(child))
			  GetVobs().Add(vob);
		  else
		  {
			  delete vob;
			  wxLogError(_("Can't load vob"));
			  return false;
		  }
	  }
	  // get pre commands
	  else if (child->GetName() == _T("pre"))
		SetPreCommands(child->GetChildren()->GetContent());
	  // get post commands
	  else if (child->GetName() == _T("post"))
		SetPostCommands(child->GetChildren()->GetContent());
	  child = child->GetNext();
  }
  return true;
}

////////////////////////////// Audio/Video ///////////////////////////////////

wxArrayString Audio::GetFormatStrings(wxString def)
{
  wxArrayString ret;
  ret.Add(def);
  ret.Add(_T("mp2"));
  ret.Add(_T("ac3"));
  ret.Add(_T("dts"));
  ret.Add(_T("pcm"));
  return ret;
}

wxArrayString Audio::GetQuantStrings(wxString def)
{
  wxArrayString ret;
  ret.Add(def);
  ret.Add(_T("16bps"));
  ret.Add(_T("20bps"));
  ret.Add(_T("24bps"));
  ret.Add(_T("drc"));
  return ret;
}

wxArrayString Audio::GetDolbyStrings(wxString def)
{
  wxArrayString ret;
  ret.Add(def);
  ret.Add(_T("surround"));
  return ret;
}

wxArrayString Audio::GetSamplerateStrings(wxString def)
{
  wxArrayString ret;
  ret.Add(def);
  ret.Add(_T("48khz"));
  ret.Add(_T("96khz"));
  return ret;
}

wxXmlNode* Audio::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("audio"));
  
  if (m_format.length())
	node->AddProperty(_T("format"), m_format);
  if (m_channels >= 0)
	node->AddProperty(_T("channels"), wxString::Format(_T("%d"), m_channels));
  if (m_quant.length())
	node->AddProperty(_T("quant"), m_quant);
  if (m_dolby.length())
	node->AddProperty(_T("dolby"), m_dolby);
  if (m_samplerate.length())
	node->AddProperty(_T("samplerate"), m_samplerate);
  if (m_lang.length())
	node->AddProperty(_T("lang"), m_lang);
  
  if (node->GetProperties() == NULL)
  {
	delete node;
	node = NULL;
  }
  return node;
}

bool Audio::PutXML(wxXmlNode* node)
{
  if (node == NULL || node->GetName() !=  _T("audio"))
	return false;
  
  wxString val;
  long lval;
  if (node->GetPropVal(_T("format"), &val))
	m_format = val;
  if (node->GetPropVal(_T("channels"), &val) && val.ToLong(&lval))
	m_channels = int(lval);
  if (node->GetPropVal(_T("quant"), &val))
	m_quant = val;
  if (node->GetPropVal(_T("dolby"), &val))
	m_dolby = val;
  if (node->GetPropVal(_T("samplerate"), &val))
	m_samplerate = val;
  if (node->GetPropVal(_T("lang"), &val))
	m_lang = val;
  
  return true;
}

wxArrayString Video::GetFormatStrings(wxString def)
{
  wxArrayString ret;
  ret.Add(def);
  ret.Add(_T("pal"));
  ret.Add(_T("ntsc"));
  return ret;
}

wxArrayString Video::GetAspectStrings(wxString def)
{
  wxArrayString ret;
  ret.Add(def);
  ret.Add(_T("4:3"));
  ret.Add(_T("16:9"));
  return ret;
}

wxArrayString Video::GetWidescreenStrings(wxString def)
{
  wxArrayString ret;
  ret.Add(def);
  ret.Add(_T("nopanscan"));
  ret.Add(_T("noletterbox"));
  return ret;
}

wxXmlNode* Video::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("video"));
  
  if (m_format.length())
	node->AddProperty(_T("format"), m_format);
  if (m_aspect.length())
	node->AddProperty(_T("aspect"), m_aspect);
  if (m_resolution.length())
	node->AddProperty(_T("resolution"), m_resolution);
  if (m_caption.length())
	node->AddProperty(_T("caption"), m_caption);
  if (m_widescreen.length())
	node->AddProperty(_T("widescreen"), m_widescreen);
  
  if (node->GetProperties() == NULL)
  {
	delete node;
	node = NULL;
  }
  return node;
}

bool Video::PutXML(wxXmlNode* node)
{
  if (node == NULL || node->GetName() !=  _T("video"))
	return false;
  
  wxString val;
  if (node->GetPropVal(_T("format"), &val))
	m_format = val;
  if (node->GetPropVal(_T("aspect"), &val))
	m_aspect = val;
  if (node->GetPropVal(_T("resolution"), &val))
	m_resolution = val;
  if (node->GetPropVal(_T("caption"), &val))
	m_caption = val;
  if (node->GetPropVal(_T("widescreen"), &val))
	m_widescreen = val;
  
  return true;
}
 
////////////////////////////////// Menus //////////////////////////////////////

wxXmlNode* Menus::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("menus"));
  if (GetAudio().GetXML(type))
	node->AddChild(GetAudio().GetXML(type));
  if (GetVideo().GetXML(type))
	node->AddChild(GetVideo().GetXML(type));
  for (int i=0; i<(int)GetCount(); i++)
	node->AddChild((*this)[i]->GetXML(type));
  return node;
}

bool Menus::PutXML(wxXmlNode* node)
{
  if (node == NULL || node->GetName() !=  _T("menus"))
	return false;
  WX_CLEAR_ARRAY(*this);
  wxXmlNode* child = node->GetChildren();
  while (child)
  {
	if (child->GetName() == _T("audio"))
	  GetAudio().PutXML(child);
	else if (child->GetName() == _T("video"))
	  GetVideo().PutXML(child);
	else if (child->GetName() == _T("pgc"))
	{
	  Pgc* pgc = new Pgc;
	  if (pgc->PutXML(child))
		Add(pgc);
	  else
	  {
		delete pgc;
		wxLogError(_("Can't load pgc"));
		return false;
	  }
	}
	child = child->GetNext();
  }
  return true;
}

////////////////////////////////// Titles ////////////////////////////////////

wxXmlNode* Titles::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("titles"));
  if (GetAudio().GetXML(type))
	node->AddChild(GetAudio().GetXML(type));
  if (GetVideo().GetXML(type))
	node->AddChild(GetVideo().GetXML(type));
  for (int i=0; i<(int)GetCount(); i++)
	node->AddChild((*this)[i]->GetXML(type));
  return node;
}

bool Titles::PutXML(wxXmlNode* node)
{
  if (node == NULL || node->GetName() !=  _T("titles"))
	return false;
  wxXmlNode* child = node->GetChildren();
  while (child)
  {
	if (child->GetName() == _T("audio"))
	  GetAudio().PutXML(child);
	else if (child->GetName() == _T("video"))
	  GetVideo().PutXML(child);
	else if (child->GetName() == _T("pgc"))
	{
	  Pgc* pgc = new Pgc;
	  if (pgc->PutXML(child))
		Add(pgc);
	  else
	  {
		delete pgc;
		wxLogError(_("Can't load pgc"));
		return false;
	  }
	}
	child = child->GetNext();
  }
  return true;
}

////////////////////////////////// Titleset //////////////////////////////////

wxXmlNode* Titleset::GetXML(DVDFileType type, DVD* dvd)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("titleset"));
  if (m_menus.Count() || type != DVDAUTHOR_XML)
	node->AddChild(m_menus.GetXML(type));
  else if (dvd->IsEmptyMenuEnabled() &&	dvd->GetVmgm().Count())
  {
	// create empty menu with post command
	Menus menus;
	Pgc* pgc = new Pgc;
	pgc->SetPostCommands(_T("jump vmgm menu;"));
	menus.Add(pgc);
	node->AddChild(menus.GetXML(type));
  }
  node->AddChild(m_titles.GetXML(type));
  return node;
}

bool Titleset::PutXML(wxXmlNode* node)
{
  if (node == NULL || node->GetName() !=  _T("titleset"))
	return false;
  wxXmlNode* child = node->GetChildren();
  while (child)
  {
	if (child->GetName() == _T("menus"))
	{
	  if (!GetMenus().PutXML(child))
	  {
		wxLogError(_("Can't load menus"));
		return false;
	  }
	}
	else if (child->GetName() == _T("titles"))
	{
	  if (!GetTitles().PutXML(child))
	  {
		wxLogError(_("Can't load titles"));
		return false;
	  }
	}
	child = child->GetNext();
  }
  return true;
}

////////////////////////////////// DVD //////////////////////////////////////

int DVD::AddTitleset()
{
  Titleset* titleset = new Titleset;
  GetTitlesets().Add(titleset);
  return GetTitlesets().Count();
}

int DVD::AddMenu(VideoFormat format, int tsi)
{
  Menu* menu = new Menu((VideoFormat)s_config.GetDefMenuFormat());
  Vob* vob = new Vob(menu);
  Pgc* pgc = new Pgc;
  pgc->GetVobs().Add(vob);
  return AddPgc(tsi, true, pgc);
}

int DVD::AddPgc(int tsi, bool menu, Pgc* pgc)
{
  if (!pgc)
	pgc = new Pgc;
  PgcArray& pgcs = GetPgcArray(tsi, menu);
  pgcs.Add(pgc);
  return pgcs.Count()-1;
}

PgcArray& DVD::GetPgcArray(int tsi, bool menus)
{
  PgcArray* pgcs = &GetVmgm();
  if (tsi>=0 && tsi<(int)GetTitlesets().Count())
  {
	Titleset* ts = GetTitlesets()[tsi];
	pgcs = menus ? (PgcArray*) &ts->GetMenus() : (PgcArray*) &ts->GetTitles();
  }
  return *pgcs;
}

Vob* DVD::GetVob(int tsi, bool isMenu, int pgci, int vobi)
{
  PgcArray& pgcs = GetPgcArray(tsi, isMenu);
  if (pgci>=(int)pgcs.Count())
	return NULL;
  if (vobi>=(int)pgcs[pgci]->GetVobs().Count())
	return NULL;
  return pgcs[pgci]->GetVobs()[vobi];
}

Vob* DVD::GetMenuVob(int tsi, int pgci)
{
  PgcArray& pgcs = GetPgcArray(tsi, true);
  if (pgci>=(int)pgcs.Count())
	return NULL;
  int vobi = 0; //TODO: find vob with menu
  if (vobi>=(int)pgcs[pgci]->GetVobs().Count())
	return NULL;
  return pgcs[pgci]->GetVobs()[vobi];
}

Menu* DVD::GetMenu(int tsi, int pgci)
{
  Vob* vob = GetMenuVob(tsi, pgci);
  if (vob)
	return vob->GetMenu();
  return NULL;
}

bool DVD::Open(wxString fname)
{
  WX_CLEAR_ARRAY(GetTitlesets());
  wxXmlDocument xml;
  if (!xml.Load(fname))
  {
	wxLogError(_("Can't load xml-file"));
	return false;
  }
  
  wxXmlNode* root = xml.GetRoot();
  if (root == NULL || root->GetName() !=  _T("dvdstyler"))
  {
	wxLogError(_("This is not a DVDStyler project file"));
	return false;
  }
  
  wxString val;
  long lval;
  if (root->GetPropVal(_T("jumppad"), &val) && val.ToLong(&lval))
	m_jumppad = int(lval);
  if (root->GetPropVal(_T("emptyMenu"), &val) && val.ToLong(&lval))
	m_emptyMenu = int(lval);
  
  wxXmlNode* child = root->GetChildren();
  while (child)
  {
	if (child->GetName() ==  _T("vmgm"))
	{
	  if (!GetVmgm().PutXML(child->GetChildren()))
	  {
		wxLogError(_("Can't load vmgm menus"));
		return false;
	  }
	}
	else if (child->GetName() ==  _T("titleset"))
	{
	  Titleset* titleset = new Titleset;
	  if (titleset->PutXML(child))
		GetTitlesets().Add(titleset);
	  else
	  {
		delete titleset;
		wxLogError(_("Can't load titleset"));
		return false;
	  }
	}
	child = child->GetNext();
  }
  return true;
}

bool DVD::Save(wxString fname)
{
  wxXmlDocument xml;
  wxXmlNode* root = new wxXmlNode(wxXML_ELEMENT_NODE, _T("dvdstyler"));
  root->AddProperty(_T("jumppad"), m_jumppad ? _T("1") : _T("0"));
  root->AddProperty(_T("emptyMenu"), m_emptyMenu ? _T("1") : _T("0"));
  wxXmlNode* vmgmNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("vmgm"));
  vmgmNode->AddChild(GetVmgm().GetXML(DVDSTYLER_XML));
  root->AddChild(vmgmNode);
  for (int i=0; i<(int)m_titlesets.GetCount(); i++)
	if (m_titlesets[i]->GetMenus().Count() || m_titlesets[i]->GetTitles().Count())
	  root->AddChild(m_titlesets[i]->GetXML(DVDSTYLER_XML, this));
  xml.SetRoot(root);
  return xml.Save(fname);
}

bool DVD::SaveDVDAuthor(wxString fname)
{
  wxXmlDocument xml;
  wxXmlNode* root = new wxXmlNode(wxXML_ELEMENT_NODE, _T("dvdauthor"));
  root->AddProperty(_T("jumppad"), m_jumppad ? _T("1") : _T("0"));
  wxXmlNode* vmgmNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("vmgm"));
  if (GetVmgm().Count())
	vmgmNode->AddChild(GetVmgm().GetXML(DVDAUTHOR_XML));
  else if (IsEmptyMenuEnabled() &&
	GetTitlesets().Count() && GetTitlesets()[0]->GetMenus().Count())
  {
	// create empty vmgm with post command
	Menus menus;
	Pgc* pgc = new Pgc;
	pgc->SetPostCommands(_T("jump titleset 1 menu;"));
	menus.Add(pgc);
	vmgmNode->AddChild(menus.GetXML(DVDAUTHOR_XML));
  }
  root->AddChild(vmgmNode);
  for (int i=0; i<(int)m_titlesets.GetCount(); i++)
	if (m_titlesets[i]->GetMenus().Count() || m_titlesets[i]->GetTitles().Count())
	  root->AddChild(m_titlesets[i]->GetXML(DVDAUTHOR_XML, this));
  xml.SetRoot(root);
  return xml.Save(fname);
}

int DVD::GetSize(bool generated)
{
  int size = 0;
  for (int menu = 0; menu<2; menu++)
  {
	for (int tsi = -int(menu); tsi<(int)GetTitlesets().Count(); tsi++)
	{
	  PgcArray& pgcs = GetPgcArray(tsi, menu);
	  for (int pgci = 0; pgci<(int)pgcs.Count(); pgci++)
	  {
		Pgc* pgc = pgcs[pgci];
		for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++)
		  size += pgc->GetVobs()[vobi]->GetSize(generated);
	  }
	}
  }
  return size;
}
