/////////////////////////////////////////////////////////////////////////////
// Name:        DVD.h
// Purpose:     The class to store a DVD Structure (Titles/Menus)
// Author:      Alex Thuering
// Created:		29.01.2003
// RCS-ID:      $Id: DVD.h,v 1.14 2005/04/13 11:06:44 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef DVDAUTHOR_H
#define DVDAUTHOR_H

#include <wx/wx.h>
#include <wx/image.h>
#include <wx/dynarray.h>
#include <wxXML/xmlhelpr.h>
#include "Menu.h"
	
class Vob
{
  public:
	Vob(wxString filename) { m_filename = filename;m_pause = 0;m_menu = NULL; }
	Vob(Menu* menu) { m_pause = -1; m_menu = menu; }
	~Vob() { if (m_menu) delete m_menu; }
	
	void SetFilename(wxString value) { m_filename = value; }
	wxString GetFilename() { return m_filename; }
	
	void SetVideoFilename(wxString value) { m_videoFilename = value; }
	wxString GetVideoFilename() { return m_videoFilename; }
	
	void SetAudioFilename(wxString value) { m_audioFilename = value; }
	wxString GetAudioFilename() { return m_audioFilename; }
	
	void SetTmpFilename(wxString value) { m_tmpFilename = value; }
	wxString GetTmpFilename() { return m_tmpFilename; }
	
	void SetChapters(wxString value) { m_chapters = value; }
	wxString GetChapters() { return m_chapters; }
	
	void SetPause(int value) { m_pause = value; }
	int GetPause() { return m_pause; }

	void SetMenu(Menu* menu) { m_menu = menu; }	
	Menu* GetMenu() { return m_menu; }
	
	wxXmlNode*	GetXML(DVDFileType type);
	bool		PutXML(wxXmlNode* node);
	
	/** returns size im Kb */
	int GetSize(bool generated = false);
	
  protected:
	wxString m_filename;
	wxString m_videoFilename;
	wxString m_audioFilename;
	wxString m_tmpFilename;
	wxString m_chapters;
	int m_pause;
	Menu* m_menu;
	static unsigned int GetFileSize(wxString filename);
};

WX_DEFINE_ARRAY(Vob*, VobArray);

class Pgc
{
  public:
	Pgc() { }
	~Pgc() { WX_CLEAR_ARRAY(m_vobs) }
	
	void SetEntry(wxString value) { m_entry = value; }
	wxString GetEntry() { return m_entry; }
	
	void SetPreCommands(wxString value) { m_pre = value; }
	wxString GetPreCommands() { return m_pre; }
	void SetPostCommands(wxString value) { m_post = value; }
	wxString GetPostCommands() { return m_post; }
	
	VobArray& GetVobs() { return m_vobs; }
	
	Menu* GetMenu();
	
	wxXmlNode*	GetXML(DVDFileType type);
	bool		PutXML(wxXmlNode* node);
	
  protected:
	VobArray m_vobs;
	wxString m_entry, m_pre, m_post;
};

WX_DEFINE_ARRAY(Pgc*, PgcArrayBase);

class Audio
{
  public:
	Audio() { m_channels = -1; }
	
	void SetFormat(wxString value) { m_format = value; }
	wxString GetFormat() { return m_format; }
	wxArrayString GetFormatStrings(wxString def);
	
	void SetChannels(int value) { m_channels = value; }
	int GetChannels() { return m_channels; }
	
	void SetQuant(wxString value) { m_quant = value; }
	wxString GetQuant() { return m_quant; }
	wxArrayString GetQuantStrings(wxString def);
	
	void SetDolby(wxString value) { m_dolby = value; }
	wxString GetDolby() { return m_dolby; }
	wxArrayString GetDolbyStrings(wxString def);
	
	void SetSamplerate(wxString value) { m_samplerate = value; }
	wxString GetSamplerate() { return m_samplerate; }
	wxArrayString GetSamplerateStrings(wxString def);
	
	void SetLang(wxString value) { m_lang = value; }
	wxString GetLang() { return m_lang; }
	
	wxXmlNode*	GetXML(DVDFileType type);
	bool		PutXML(wxXmlNode* node);
	
  protected:
	wxString m_format;
	int m_channels;
	wxString m_quant;
	wxString m_dolby;
	wxString m_samplerate;
	wxString m_lang;
};

class Video
{
  public:
	void SetFormat(wxString value) { m_format = value; }
	wxString GetFormat() { return m_format; }
	wxArrayString GetFormatStrings(wxString def);
	
	void SetAspect(wxString value) { m_aspect = value; }
	wxString GetAspect() { return m_aspect; }
	wxArrayString GetAspectStrings(wxString def);
	
	void SetResolution(wxString value) { m_resolution = value; }
	wxString GetResolution() { return m_resolution; }
	
	void SetCaption(wxString value) { m_caption = value; }
	wxString GetCaption() { return m_caption; }
	
	void SetWidescreen(wxString value) { m_widescreen = value; }
	wxString GetWidescreen() { return m_widescreen; }
	wxArrayString GetWidescreenStrings(wxString def);
	
	wxXmlNode*	GetXML(DVDFileType type);
	bool		PutXML(wxXmlNode* node);
	
  protected:
	wxString m_format;
	wxString m_aspect;
	wxString m_resolution;
	wxString m_caption;
	wxString m_widescreen;
};

class PgcArray: public PgcArrayBase
{
  public:
	Audio& GetAudio() { return m_audio; }
	Video& GetVideo() { return m_video; }
  
  protected:
	Audio m_audio;
	Video m_video;
};

class Menus: public PgcArray
{
  public:
	wxXmlNode*	GetXML(DVDFileType type);
	bool		PutXML(wxXmlNode* node);
};

class Titles: public PgcArray
{
  public:
	wxXmlNode* GetXML(DVDFileType type);
	bool	   PutXML(wxXmlNode* node);
};

class DVD;
class Titleset
{
  public:
	Titleset() { WX_CLEAR_ARRAY(m_menus); WX_CLEAR_ARRAY(m_titles); }
	
	Menus& GetMenus() { return m_menus; }
	Titles& GetTitles() { return m_titles; }
	
	wxXmlNode* GetXML(DVDFileType type, DVD* dvd);
	bool	   PutXML(wxXmlNode* node);
	
  protected:
	Menus m_menus;
	Titles m_titles;
};

WX_DEFINE_ARRAY(Titleset*, TitlesetArray);

class DVD
{
  public:
	DVD() { m_jumppad = true; m_emptyMenu = true; }
	~DVD() { WX_CLEAR_ARRAY(m_titlesets); WX_CLEAR_ARRAY(m_vmgm); }
	
	/** returns true if creation of jumppads enabled */
	bool IsJumppadsEnabled() { return m_jumppad; }
	/** enables creation of jumppads */
	void EnableJumppads(bool enable = true) { m_jumppad = enable; }
	
	/** returns true if creation of empty menu enabled */
	bool IsEmptyMenuEnabled() { return m_emptyMenu; }
	/** enables creation of empty menu */
	void EnableEmptyMenu(bool enable = true) { m_emptyMenu = enable; }
	
	/** returns titlesets */
	TitlesetArray& GetTitlesets() { return m_titlesets; }
	/** returns vmgm menus */
	Menus& GetVmgm() { return m_vmgm; }
	
	/** returns index of new titleset (tsi) */
	int AddTitleset();
	/** returns index of new pgc */
	int AddMenu(VideoFormat format, int tsi = -1);
	/** returns index of new pgc */
	int AddPgc(int tsi, bool menu, Pgc* pgc = NULL);
	/** returns pgcs */
	PgcArray& GetPgcArray(int tsi, bool menus);
	/** returns vob from given titleset, pgc */
	Vob* GetVob(int tsi, bool isMenu, int pgci, int vobi);
	/** returns vob with menu from given titleset and pgc */
	Vob* GetMenuVob(int tsi, int pgci);
	/** returns menu from titleset tsi und pgc pgci */
	Menu* GetMenu(int tsi, int pgci);
	
	/** returns id for specified vob */
	static inline int MakeId(int tsi, int pgci, int vobi, bool menu)
	{ return ((tsi+1)<<24) + ((pgci*2+(menu ? 0 : 1))<<8) + vobi; }
	static inline int GetTsi(int id)  { return (id>>24)-1; }
	static inline int GetPgci(int id) { return ((id>>8) & 0xFFFF) / 2; }
	static inline bool IsMenu(int id) { return !(((id>>8) & 0xFFFF) % 2); }
	static inline int GetVobi(int id) { return id & 0xFF; }
	
	bool Open(wxString fname);
	bool Save(wxString fname);
	bool SaveDVDAuthor(wxString fname);
	/** returns size im Kb */
	int GetSize(bool generated = false);
	
  protected:
	bool m_jumppad;
	bool m_emptyMenu;
	TitlesetArray m_titlesets;
	Menus m_vmgm;
};

#endif //DVDAUTHOR_H
