/////////////////////////////////////////////////////////////////////////////
// Name:        MenuBox.h
// Purpose:     The widget to edit a DVD Menu
// Author:      Alex Thuering
// Created:		11.10.2003
// RCS-ID:      $Id: MenuBox.h,v 1.14 2004/12/09 09:16:37 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////
 
#ifndef MENUBOX_H
#define MENUBOX_H

#include <wxVillaLib/ImageBox.h>
#include "DVD.h"

typedef enum TransformType
{ ttOut, ttIn, ttTL, ttTR, ttBL, ttBR };

class MenuBox: public wxImageBox
{
  public:
    MenuBox(wxWindow *parent, wxWindowID id = -1);
	
	void  SetMenu(DVD* dvd, Menu* menu, int tsi, int pgci);
	Menu* GetMenu() { return m_menu; }
	DVD*  GetDVD()	{ return m_dvd;	}
	void  UpdateMenu();
	
	bool SetBackground(wxString fname);
	void SetBackgroundColour(wxColour colour);
	bool AddImage(wxString fname, int x = 0, int y = 0);
	bool AddText(wxString text, int x = 0, int y = 0);
	bool AddButton(wxString fname, int x = 0, int y = 0);
	
	int  GetSelected() { return m_selected; }
	void SetSelected(int index) { m_selected = index; }
	
	int  GetPointed() { return m_pointed; }
	void SetPointed(int index) { m_pointed = index; }
	
	int  GetGrid() { return m_grid; }
	void SetGrid(int value) { m_grid = value; }

  protected:
	DVD*  m_dvd;
	Menu* m_menu;
	int m_tsi;
	int m_pgci;
	wxMenu* m_objMenu;
	wxMenu* m_gridMenu;
	wxMenu* m_addMenu;
	wxPoint m_menuPos;
	int m_selected;
	int m_pointed;
	int m_grid;
	virtual void Paint();
	
    TransformType m_tt;
    int m_oldx, m_oldy;
    TransformType GetTransformType(int x, int y);
    bool MoveObject(int& x, int& y);
    bool MoveObjectInt(int& x, int& y);
	bool ResizeObject(int x, int y, TransformType transformType = ttBR);
    bool ResizeObjectInt(int x, int y, TransformType transformType = ttBR);
	int GetObject(int x, int y);
	
	void OnMouseLeftButton(wxMouseEvent &event);
    void OnMouseMove(wxMouseEvent &event);
	void OnMouseRightButton(wxMouseEvent &event);
	void OnMouseDClick(wxMouseEvent& WXUNUSED(event));
	
	void OnKeyDown(wxKeyEvent& event);
	
	void OnObjectCut(wxCommandEvent& WXUNUSED(event));
	void OnObjectCopy(wxCommandEvent& WXUNUSED(event));
	void OnObjectPaste(wxCommandEvent& WXUNUSED(event));
	void OnUpdateUIObjectPaste(wxUpdateUIEvent& event);
	void OnObjectDelete(wxCommandEvent& WXUNUSED(event));
	void OnProperties(wxCommandEvent& WXUNUSED(event));
	void OnGridMenu(wxCommandEvent& event);
	void OnAddText(wxCommandEvent& WXUNUSED(event));
	
    DECLARE_EVENT_TABLE()
};

BEGIN_DECLARE_EVENT_TYPES()
  DECLARE_EVENT_TYPE(EVT_COMMAND_MENUBOX_OBJECT_POINTED, 3305)
END_DECLARE_EVENT_TYPES()

#define EVT_MENUBOX_OBJECT_POINTED(id, fn)\
 DECLARE_EVENT_TABLE_ENTRY(EVT_COMMAND_MENUBOX_OBJECT_POINTED, id,\
 wxID_ANY,\
 (wxObjectEventFunction) (wxEventFunction) (wxCommandEventFunction)& fn, NULL),

#endif // MENUBOX_H
