/////////////////////////////////////////////////////////////////////////////
// Name:        ProgressDlg.cpp
// Purpose:     Progress of generation/burning dialog
// Author:      Alex Thuering
// Created:     14.08.2004
// RCS-ID:      $Id: ProgressDlg.cpp,v 1.23 2005/05/17 12:42:25 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "ProgressDlg.h"
#include "Config.h"
#include "MPEG.h"
#include <wxVillaLib/PipeExecute.h>
#include <wxVillaLib/utils.h>
#include <wx/dir.h>
#include <wx/filename.h>

#define DATA_FILE(fname) wxFindDataFile(_T("data") + wxString(wxFILE_SEP_PATH) + fname)

////////////////////////// Process Execute ///////////////////////////////////

#define MAX_WARNINGS 10

class ProcessExecute: public wxPipeExecute
{
  public:
	ProcessExecute(ProgressDlg* processDlg): m_processDlg(processDlg) {}
	
	virtual void ProcessOutput(wxString line)
	{
	  m_processDlg->AddDetailText(line + _T("\n"));
	}
	
	virtual bool IsCanceled()
	{
	  return m_processDlg->IsCanceled();
	}
	
  protected:
	ProgressDlg* m_processDlg;
};

class ProcentExecute: public ProcessExecute
{
  public:
	ProcentExecute(ProgressDlg* processDlg):
	  ProcessExecute(processDlg), m_percent(0)
	{
	  m_initDetailStep = m_processDlg->GetDetailStep();
	}
	
	virtual void ProcessOutput(wxString line)
	{
	  if (line.Find(wxT('%'))>0)
	  {
		long percent = 0;
		wxString percentStr = line.BeforeLast(wxT('%')).Strip().AfterLast(wxT(' '));
		percentStr = percentStr.BeforeFirst(wxT('.')).BeforeFirst(wxT(','));
		if (percentStr.ToLong(&percent))
		{
		  m_processDlg->SetDetailStep(m_initDetailStep + (int)percent);
		  if (percent >= m_percent)
			m_percent += 5;
		  else
			return;
		}
	  }
	  m_processDlg->AddDetailText(line + _T("\n"));
	}

  protected:
	int m_initDetailStep;
	int m_percent;
};

class DVDAuthorExecute: public ProcentExecute
{
  public:
	DVDAuthorExecute(ProgressDlg* processDlg, int totalSize): ProcentExecute(processDlg),
	  m_totalSize(totalSize), m_warnings(0), m_warnStep(1), m_dvdauthorStep(0)
	{
	  if (m_totalSize == 0)
		m_totalSize++;
	}
	
	virtual void ProcessOutput(wxString line)
	{
	  if (m_dvdauthorStep)
		return ProcentExecute::ProcessOutput(line);
	  if (line.Mid(0,11) == _T("STAT: fixed"))
	  {
		m_dvdauthorStep++;
		m_initDetailStep += 200;
		m_processDlg->SetDetailStep(m_initDetailStep);
	  }
	  else if (line.Mid(0,10) == _T("STAT: VOBU"))
	  {
		long size = 0;
		wxString sizeStr = line.BeforeLast(wxT(',')).AfterLast(wxT(' '));
		if (sizeStr.Mid(sizeStr.Length()-2) == _T("MB") &&
			sizeStr.Remove(sizeStr.Length()-2).ToLong(&size))
		  m_processDlg->SetDetailStep(m_initDetailStep + (int)size*200/m_totalSize);
	  }
	  else if (line.Mid(0,5) == _T("WARN:"))
	  {
		m_warnings++;
		if (m_warnings > m_warnStep*10)
		  m_warnStep = m_warnStep*10;
		else if (m_warnings % m_warnStep != 0)
		  return;
	  }
	  m_processDlg->AddDetailText(line + _T("\n"));
	}

  protected:
	int m_totalSize;
	int m_warnings;
	int m_warnStep;
	int m_dvdauthorStep;
};

/////////////////////////// Process Dialog ///////////////////////////////////

BEGIN_EVENT_TABLE(ProgressDlg, wxDialog)
  EVT_BUTTON(wxID_CANCEL, ProgressDlg::OnCancel)
  EVT_BUTTON(HIDE_BT_ID, ProgressDlg::OnHideDetails)
  EVT_BUTTON(ICONIZE_BT_ID, ProgressDlg::OnMinimize)
END_EVENT_TABLE()

ProgressDlg::ProgressDlg(wxWindow* parent):
  wxDialog(parent, -1, wxEmptyString, wxDefaultPosition, wxDefaultSize,
  wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxTHICK_FRAME)
{
    // begin wxGlade: ProgressDlg::ProgressDlg
    m_summaryLabel = new wxStaticText(this, -1, _("Summary:"));
    m_summaryText = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH);
    m_summaryGauge = new wxGauge(this, -1, 10, wxDefaultPosition, wxDefaultSize, wxGA_HORIZONTAL|wxGA_PROGRESSBAR|wxGA_SMOOTH);
    m_detailsGauge = new wxGauge(this, -1, 10, wxDefaultPosition, wxDefaultSize, wxGA_HORIZONTAL|wxGA_PROGRESSBAR|wxGA_SMOOTH);
    m_detailsLabel = new wxStaticText(this, -1, _("Details:"));
    m_detailsText = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH);
    m_detailsBt = new wxButton(this, HIDE_BT_ID, _("Hide details"));
    m_minimizeBt = new wxButton(this, ICONIZE_BT_ID, _("Minimize"));
    m_cancelBt = new wxButton(this, wxID_CANCEL, _("Cancel"));

    set_properties();
    do_layout();
    // end wxGlade
    m_cancel = false;
}

void ProgressDlg::set_properties()
{
    // begin wxGlade: ProgressDlg::set_properties
    SetTitle(_("Generate DVD"));
    SetSize(wxSize(600, 600));
    // end wxGlade
	
	m_detailsBtLabel = m_detailsBt->GetLabel();
	m_detailsBt->SetLabel(_T("<< ") + m_detailsBtLabel);
}

void ProgressDlg::do_layout()
{
    // begin wxGlade: ProgressDlg::do_layout
    wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer* btSizer = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer* panelSizer = new wxBoxSizer(wxVERTICAL);
    panelSizer->Add(m_summaryLabel, 0, wxBOTTOM, 2);
    panelSizer->Add(m_summaryText, 1, wxBOTTOM|wxEXPAND, 8);
    panelSizer->Add(m_summaryGauge, 0, wxBOTTOM|wxEXPAND, 4);
    panelSizer->Add(m_detailsGauge, 0, wxBOTTOM|wxEXPAND, 4);
    panelSizer->Add(m_detailsLabel, 0, wxBOTTOM, 2);
    panelSizer->Add(m_detailsText, 1, wxEXPAND, 0);
    mainSizer->Add(panelSizer, 1, wxLEFT|wxRIGHT|wxTOP|wxEXPAND, 10);
    btSizer->Add(m_detailsBt, 0, wxALIGN_CENTER_VERTICAL, 0);
    btSizer->Add(20, 20, 1, wxEXPAND, 0);
    btSizer->Add(m_minimizeBt, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 8);
    btSizer->Add(m_cancelBt, 0, 0, 0);
    mainSizer->Add(btSizer, 0, wxALL|wxEXPAND, 10);
    SetAutoLayout(true);
    SetSizer(mainSizer);
    Layout();
    Centre();
    // end wxGlade
	m_panelSizer = panelSizer;
#ifdef __WXMSW__
	m_minimizeBt->Hide();
#endif
}

void ProgressDlg::OnHideDetails(wxCommandEvent& WXUNUSED(event))
{
  if (m_detailsText->IsShown())
  {
	m_detailsLabel->Hide();
	m_detailsText->Hide();
	m_detailsText->Freeze();
	m_panelSizer->Remove(m_detailsLabel);
	m_panelSizer->Remove(m_detailsText);
	int height = m_detailsLabel->GetSize().GetY() +
	  m_detailsText->GetSize().GetY() + 2;
	SetSize(GetSize().GetX(), GetSize().GetY() - height);
	m_detailsBt->SetLabel(_("Show details") + wxString(_T(" >>")));
  }
  else
  {
	m_detailsLabel->Show();
	m_detailsText->Show();
	m_detailsText->Thaw();
	m_panelSizer->Insert(4, m_detailsLabel, 0, wxBOTTOM, 2);
	m_panelSizer->Insert(5, m_detailsText, 1, wxEXPAND, 0);
	int height = m_detailsLabel->GetSize().GetY() +
	  m_detailsText->GetSize().GetY() + 2;
	SetSize(GetSize().GetX(), GetSize().GetY() + height);
	m_detailsBt->SetLabel(_T("<< ") + m_detailsBtLabel);
  }
}

void ProgressDlg::OnMinimize(wxCommandEvent& WXUNUSED(event))
{
  ((wxFrame*)GetParent())->Iconize();
}

void ProgressDlg::AddSummaryMsg(const wxString& message,
  const wxString& details, const wxColour& colour)
{
  m_summaryText->SetDefaultStyle(wxTextAttr(colour.Ok() ? colour : *wxBLACK));
  m_summaryText->AppendText(message + _T("\n"));
  m_summaryText->ShowPosition(m_summaryText->GetLastPosition());
  AddDetailMsg(details.length() ? details : message, colour.Ok() ? colour : *wxBLACK);
}

void ProgressDlg::AddDetailMsg(const wxString& message, const wxColour& colour)
{
  if (m_cancel)
	return;
  if (colour.Ok())
	m_detailsText->SetDefaultStyle(wxTextAttr(colour));
  AddDetailText(message + _T("\n"));
  m_detailsText->SetDefaultStyle(wxTextAttr(wxColour(64,64,64)));
}

void ProgressDlg::AddDetailText(const wxString& text)
{
  m_detailsText->AppendText(text);
  m_detailsText->ShowPosition(m_detailsText->GetLastPosition());
}

void ProgressDlg::UpdateSummary()
{
  if (m_detailsGauge->GetRange() == 0)
	return;
  int value = m_detailsGauge->GetValue()*100/m_detailsGauge->GetRange();
  m_summaryGauge->SetValue(m_lastSummary + value);
}

void ProgressDlg::Failed(const wxString& message)
{
  AddSummaryMsg(_("Failed"), message, *wxRED);
}

void ProgressDlg::FailedExec(const wxString& command)
{
  Failed(_("Error executing of command: ") + command);
}

void ProgressDlg::OnCancel(wxCommandEvent& WXUNUSED(event))
{
  if (!m_cancel)
  {
	AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
    End();
  }
  else
    EndModal(wxID_OK);
}

void ProgressDlg::End()
{
  m_cancelBt->SetLabel(_("Close"));
  m_cancel = true;
}

bool ProgressDlg::IsCanceled()
{
  wxYield();
  return m_cancel;
}

void ProgressDlg::Start(BurnDlg* burnDlg, DVD* dvd)
{
  m_winDisabler = new wxWindowDisabler(this);
  Show();
  Run(burnDlg, dvd);
  End();
  delete m_winDisabler;
  m_winDisabler = NULL;
  ShowModal();
}

void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
{
  if (IsCanceled())
	return;
  
  wxString tmpDir = burnDlg->GetTempDir();
  if (tmpDir.Last() != wxFILE_SEP_PATH)
    tmpDir += wxFILE_SEP_PATH;
  
  // menus
  wxArrayPtrVoid menuVobs;
  int menuDetailsSteps = 0;
  for (int tsi = -1; tsi<(int)dvd->GetTitlesets().Count(); tsi++)
  {
	PgcArray& pgcs = dvd->GetPgcArray(tsi, true);
	for (int pgci = 0; pgci<(int)pgcs.Count(); pgci++)
	{
	  Pgc* pgc = pgcs[pgci];
	  if (pgc->GetVobs().Count() == 0)
	    continue;
	  wxString menuFile = tmpDir + wxString::Format(_T("menu%d-%d.mpg"), tsi+1, pgci);
	  Vob* vob = pgc->GetVobs()[0];
	  vob->SetTmpFilename(menuFile);
	  menuDetailsSteps += 10;
	  if (vob->GetMenu()->HasVideoBackground())
	  {
		if (!MPEG::HasNavPacks(vob->GetMenu()->GetBackground()))
		  menuDetailsSteps += 200; // fixvob(200)
		menuDetailsSteps += 100; // spumux(100)
	  }
	  else
		menuDetailsSteps += 150; // jpeg->mpg(50) + mplex(50) + spumux(50)
	  menuVobs.Add(vob);
	}
  }
  
  // titles
  wxArrayPtrVoid titleVobs;
  int titleDetailsSteps = 0;
  for (int tsi = 0; tsi<(int)dvd->GetTitlesets().Count(); tsi++)
  {
	Titleset* ts = dvd->GetTitlesets()[tsi];
	for (int pgci = 0; pgci<(int)ts->GetTitles().Count(); pgci++)
	{
	  Pgc* pgc = ts->GetTitles()[pgci];
	  for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++)
	  {
		Vob* vob = pgc->GetVobs()[vobi];
		vob->SetTmpFilename(_T(""));
		if (vob->GetFilename().length())
		{
		  if (MPEG::HasNavPacks(vob->GetFilename()))
			continue;
		  titleDetailsSteps += 200;
		}
		else
		{
		  if (!vob->GetVideoFilename().length() || !vob->GetAudioFilename().length())
			continue;
		  titleDetailsSteps += 100;
		}
		vob->SetTmpFilename(tmpDir + wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
		titleVobs.Add(vob);
	  }
	}
  }
  
  // BEGIN
  int summarySteps = 1;
  if (menuVobs.Count()>0)
	summarySteps++;
  if (titleVobs.Count()>0)
	summarySteps++;
  if (burnDlg->DoPreview())
	summarySteps++;
  if (burnDlg->DoCreateIso() || burnDlg->DoBurn())
	summarySteps++;
  if (burnDlg->DoBurn() && burnDlg->DoFormat())
	summarySteps++;
  SetSummaryRange(summarySteps);
  
  // 1. Prepare
  AddSummaryMsg(_("Prepare"));
  AddDetailMsg(_("Cleaning temporaly directory"));
  if (wxDir::Exists(tmpDir) && wxFindFirstFile(tmpDir + _T("*"), wxDIR|wxFILE).length())
  {
    if (wxMessageBox(_("The temp directory isn't empty.\nThis direcory must be cleaned.\nContinue?"),
	    _("Burn"), wxYES_NO|wxICON_QUESTION, this) != wxYES)
	{
	  AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
	  return;
	}
    if (tmpDir.length()<=3 || !DeleteTempDir(tmpDir))
    {
      Failed(wxString::Format(_("Can't remove directory '%s'"), tmpDir.c_str()));
      return;
    }
  }
  // create tmp dir
  if (!wxDir::Exists(tmpDir) && !wxMkdir(tmpDir))
  {
	Failed(wxString::Format(_("Can't create directory '%s'"), tmpDir.c_str()));
    return;
  }
        
  // 2. Generate menus
  if (menuVobs.Count()>0)
  {
	if (IsCanceled())
	  return;
	AddSummaryMsg(_("Generating menus"));
	SetDetailRange(menuDetailsSteps);
	for (int i=0; i<(int)menuVobs.Count(); i++)
	{
	  AddDetailMsg(wxString::Format(_("Generating menu %d"), i+1));
	  Vob* vob = (Vob*)menuVobs[i];
	  if (!GenerateMenu(vob->GetMenu(), vob->GetTmpFilename()))
		return;
	}
	IncSummaryStep();
  }
  
  // Test all mpeg-files
  if (titleVobs.Count()>0)
  {
	if (IsCanceled())
	  return;
	AddSummaryMsg(_("Fixing MPEG-files"));
	SetDetailRange(titleDetailsSteps);
	for (int i=0; i<(int)titleVobs.Count(); i++)
	{
	  Vob* vob = (Vob*)titleVobs[i];
	  if (vob->GetFilename().length())
	  {
		if (!FixVob(vob->GetFilename(), vob->GetTmpFilename()))
		  return;
	  }
	  else
	  {
		if (!MakeVob(vob->GetVideoFilename(), vob->GetAudioFilename(),
			  vob->GetTmpFilename()))
		  return;
	  }
	}
	IncSummaryStep();
  }
  
  /// Generate dvd structure ///
  if (IsCanceled())
    return;
  wxString dvdauthFile = tmpDir + _T("dvdauthor.xml");
  dvd->SaveDVDAuthor(dvdauthFile);
  AddSummaryMsg(_("Generating DVD"));
  SetDetailRange(300);
  wxString cmd = s_config.GetDvdauthorCmd();
  cmd.Replace(_T("$FILE_CONF"), dvdauthFile);
  cmd.Replace(_T("$DIR"), tmpDir);
  DVDAuthorExecute dvdauthorExec(this, dvd->GetSize(true)/1024);
  if (!dvdauthorExec.Execute(cmd))
  {
    FailedExec(cmd);
    return;
  }
  // remove temp files
  wxDir d(tmpDir);
  wxString fname;
  if (s_config.GetRemoveTempFiles())
  {
	while (d.GetFirst(&fname, _T("*.vob"), wxDIR_FILES))
	  if(!DeleteFile(tmpDir + fname))
	    return;
	while (d.GetFirst(&fname, _T("menu*"), wxDIR_FILES))
	  if (!DeleteFile(tmpDir + fname))
	    return;
	if (!DeleteFile(dvdauthFile))
	    return;
  }
  SetDetailStep(300);
  IncSummaryStep();

  if (burnDlg->DoPreview())
  {
    if (IsCanceled())
      return;
    AddSummaryMsg(_("Start preview"));
	SetDetailRange(100);
    wxString cmd = s_config.GetPreviewCmd();
    cmd.Replace(_T("$DIR"), tmpDir);
    if (!Exec(cmd))
      FailedExec(cmd);
    if (burnDlg->DoBurn() && wxMessageBox(_("Burn this video to dvd?"),
         _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
	{
	  AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
      return;
	}
    if (burnDlg->DoCreateIso() && wxMessageBox(_("Create iso image of this video?"),
         _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
    {
	  AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
      return;
	}
	SetDetailStep(100);
	IncSummaryStep();
  }
  
  if (burnDlg->DoCreateIso())
  {
    if (IsCanceled())
      return;
    AddSummaryMsg(_("Creating ISO image"));
	SetDetailRange(100);
    wxString cmd = s_config.GetIsoCmd();
    cmd.Replace(_T("$DIR"), tmpDir);
    cmd.Replace(_T("$FILE"), burnDlg->GetIsoFile());
	ProcentExecute exec(this);
    if (!exec.Execute(cmd))
    {
      FailedExec(cmd);
      return;
    }
    // remove tmp dir
  	DeleteTempDir(tmpDir);
	SetDetailStep(100);
	IncSummaryStep();
  }
  
  if (burnDlg->DoBurn() && burnDlg->DoFormat())
  {
    if (IsCanceled())
      return;
    AddSummaryMsg(_("Formating DVD-RW"));
    while (1)
    {
	  SetDetailRange(100);
      wxString cmd = s_config.GetFormatCmd();
      cmd.Replace(_T("$DEV"), burnDlg->GetDevice());
      if (!Exec(cmd))
      {
	    int repeat = wxMessageBox(_("Formating DVD-RW failed. Try again?"),
	      _("Burn"), wxYES_NO|wxCANCEL|wxICON_QUESTION, this);
        if (repeat == wxYES)
	      continue;
	    else if (repeat == wxNO)
		{
		  AddSummaryMsg(_("-> skipped <-"), wxEmptyString, wxColour(128, 64, 64));
	      break;
		}
	    else
		{
		  FailedExec(cmd);
	      return;
		}
      }
      break;
    }
	SetDetailStep(100);
	IncSummaryStep();
  }
  
  if (burnDlg->DoBurn())
  {
    if (IsCanceled())
      return;
    AddSummaryMsg(_("Burning"));
	SetDetailRange(100);
    wxString cmd = s_config.GetBurnCmd();
    cmd.Replace(_T("$DIR"), tmpDir);
    cmd.Replace(_T("$DEV"), burnDlg->GetDevice());
	wxString speedStr;
	if (burnDlg->GetSpeed() > 0)
	  speedStr = wxString::Format(_T("-speed=%d"), burnDlg->GetSpeed());
	cmd.Replace(_T("$SPEEDSTR"), speedStr);
	ProcentExecute exec(this);
    if (!exec.Execute(cmd))
    {
      FailedExec(cmd);
      return;
    }
    // remove tmp tmpDir
    DeleteTempDir(tmpDir);
	SetDetailStep(100);
	IncSummaryStep();
  }
  
  if (IsCanceled())
    return;
  
  if (burnDlg->DoBurn())
    AddSummaryMsg(_("Burning was successful."), wxEmptyString, wxColour(0,128,0));
  else
    AddSummaryMsg(_("Generating was successful."), wxEmptyString, wxColour(0,128,0));
  IncSummaryStep();
}

bool ProgressDlg::MakeVob(const wxString& videoFile, const wxString& audioFile,
  const wxString& vobFile)
{
  if (IsCanceled())
	return false;
  AddSummaryMsg(wxString::Format(_("Make Vob-file from '%s' and '%s'"),
	videoFile.c_str(), audioFile.c_str()));
  // remove old
  wxString cmd = s_config.GetMplexCmd();
  cmd.Replace(_T("$FILE_VIDEO"), videoFile);
  cmd.Replace(_T("$FILE_AUDIO"), audioFile);
  cmd.Replace(_T("$FILE_OUT"), vobFile);
  if (!Exec(cmd))
  {
	FailedExec(cmd);
	return false;
  }
  return true;
}

bool ProgressDlg::FixVob(const wxString& mpegFile, const wxString& vobFile)
{
  if (IsCanceled())
	return false;
  wxString tmpDir = vobFile.BeforeLast(wxFILE_SEP_PATH) + wxFILE_SEP_PATH;
  wxString tmpFile = vobFile.AfterLast(wxFILE_SEP_PATH).BeforeLast(wxT('.'));
  AddDetailMsg(_("Fix MPEG-file: ") + mpegFile);
  // remove old
  wxDir d(tmpDir);
  wxString fname;
  while (d.GetFirst(&fname, tmpFile + _T("*"), wxDIR_FILES))
	if (!DeleteFile(tmpDir + fname))
	  return false;
  wxString cmd = s_config.GetDemplexCmd();
  cmd.Replace(_T("$FILE_IN"), mpegFile);
  cmd.Replace(_T("$FILES_OUT"), tmpDir + tmpFile);
  ProcentExecute exec(this);
  if (!exec.Execute(cmd))
  {
	FailedExec(cmd);
	return false;
  }
  cmd = s_config.GetMplexCmd();
  d.GetFirst(&fname, tmpFile + _T("*"), wxDIR_FILES);
  cmd.Replace(_T("$FILE_VIDEO"), tmpDir + fname);
  d.GetNext(&fname);
  cmd.Replace(_T("$FILE_AUDIO"), tmpDir + fname);
  cmd.Replace(_T("$FILE_OUT"), vobFile);
  if (!Exec(cmd))
  {
	FailedExec(cmd);
	return false;
  }
  IncDetailStep(99);
  while (d.GetFirst(&fname, tmpFile + _T("-*"), wxDIR_FILES))
	if (!DeleteFile(tmpDir + fname))
	  return false;
  IncDetailStep();
  return true;
}

bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile)
{
  if (IsCanceled())
	return false;
  wxString bgFile  = menuFile + _T("_bg.jpg");
  wxString bgvFile = menuFile + _T("_bg.m2v");
  wxString bgmFile = menuFile + _T("_bg.mpg");
  wxString btFile  = menuFile + _T("_buttons.png");
  wxString hlFile  = menuFile + _T("_highlight.png");
  wxString selFile = menuFile + _T("_select.png");
  wxString spuFile = menuFile + _T("_spumux.xml");
  wxString silenceFile = DATA_FILE(_T("silence.mp2"));
  
  bool videoMenu = menu->HasVideoBackground();
  wxYield();
  
  // save background
  AddDetailMsg(_("Prepare"));
  wxImage img;
  if (!videoMenu)
  {
    img = menu->GetImage(mdBACKGROUND).ConvertToImage();
    img.SetOption(_T("quality"), 100);
    img.SaveFile(bgFile);
  }
  // save buttons
  menu->SetTransparentColor();
  img = menu->GetImage(mdBUTTONS_NORMAL).ConvertToImage();
  img.SaveFile(btFile);
  img = menu->GetImage(mdBUTTONS_HIGHLIGHTED).ConvertToImage();
  img.SaveFile(hlFile);
  img = menu->GetImage(mdBUTTONS_SELECTED).ConvertToImage();
  img.SaveFile(selFile);
  // save spumux
  menu->SaveSpumux(spuFile, btFile, hlFile, selFile);
  IncDetailStep(10);
  
  // convert jpeg to mpeg
  if (videoMenu)
  {
	if (!MPEG::HasNavPacks(menu->GetBackground()))
	{
	  int detailStep = GetDetailStep();
	  if (!FixVob(menu->GetBackground(), bgmFile))
		return false;
	  SetDetailStep(detailStep+200);
	}
	else
	  bgmFile = menu->GetBackground();
  }
  else
  {
    AddDetailMsg(_("Converting jpg to mpeg"));
    wxString videoNorm;
    wxString frameRate = s_config.GetFrameRatePAL();
    if (menu->GetVideoFormat() == vfNTSC)
    {
	  videoNorm = _T(" --video-norm n --frame-rate 4");
	  frameRate = s_config.GetFrameRateNTSC();
    }
    wxString cmd = s_config.GetJpeg2MpegCmd();
    cmd.Replace(_T("$FRAME_RATE"), frameRate);
    cmd.Replace(_T("$VIDEO_NORM"), videoNorm);
	cmd.Replace(_T("$BITRATE"),
	  wxString::Format(_T("%d"), s_config.GetMenuVideoBitrate()));
    cmd.Replace(_T("$FILE_IN"), bgFile);
    cmd.Replace(_T("$FILE_OUT"), bgvFile);
    long ret = Exec(cmd);
    if (!ret)
    {
	  FailedExec(cmd);
	  return false;
    }
	if (s_config.GetRemoveTempFiles())
	  if (!DeleteFile(bgFile))
	    return false;
	IncDetailStep(50);
    
    if (IsCanceled())
      return false;
    AddDetailMsg(_("Multiplexing audio and video"));
  	cmd = s_config.GetMplexCmd();
    cmd.Replace(_T("$FILE_VIDEO"), bgvFile);
    cmd.Replace(_T("$FILE_AUDIO"), silenceFile);
    cmd.Replace(_T("$FILE_OUT"), bgmFile);
    if (!Exec(cmd))
    {
	  FailedExec(cmd);
	  return false;
    }
	if (s_config.GetRemoveTempFiles())
	  if (!DeleteFile(bgvFile))
	    return false;
	IncDetailStep(50);
  }
  
  //spumux
  if (IsCanceled())
    return false;
  AddDetailMsg(_("Multiplexing subtitles (buttons) into mpeg"));
  wxString cmd = s_config.GetSpumuxCmd();
  cmd.Replace(_T("$FILE_CONF"), spuFile);
  if (!Exec(cmd, bgmFile, menuFile))
  {
    FailedExec(cmd);
    return false;
  }
  if (s_config.GetRemoveTempFiles())
  {
	if ((!videoMenu || bgmFile != menu->GetBackground()) &&
		!DeleteFile(bgmFile))
	  return false;
	if (!DeleteFile(btFile) ||
        !DeleteFile(hlFile) ||
        !DeleteFile(selFile) ||
        !DeleteFile(spuFile))
      return false;
  }
  IncDetailStep(videoMenu ? 100 : 50);
  
  wxYield();
  return true;
}

bool ProgressDlg::DeleteFile(wxString fname)
{
  if (!wxRemoveFile(fname))
  {
    Failed(wxString::Format(_("Can't remove file '%s'"), fname.c_str()));
    return false;
  }
  return true;
}

bool ProgressDlg::DeleteDir(wxString dir, bool subdirs)
{
  if (dir.Last() != wxFILE_SEP_PATH)
	dir += wxFILE_SEP_PATH;
  wxDir d(dir);
  wxString fname;
  while (subdirs && d.GetFirst(&fname, wxEmptyString, wxDIR_DIRS))
    if (!DeleteDir(dir + fname, true))
      return false;
  while (d.GetFirst(&fname, wxEmptyString, wxDIR_FILES | wxDIR_HIDDEN))
    if (!DeleteFile(dir + fname))
      return false;
  d.Open(wxGetHomeDir());
  wxLogNull log;
  wxRmdir(dir);
  return true;
}

bool ProgressDlg::DeleteTempDir(wxString dir)
{
  if (dir.Last() != wxFILE_SEP_PATH)
	dir += wxFILE_SEP_PATH;
  wxDir d(dir);
  wxString fname;
  d.GetFirst(&fname, wxEmptyString, wxDIR_DIRS);
  while (1)
  {
	if (fname.Lower() == _T("video_ts") || fname.Lower() == _T("audio_ts"))
	  if (!DeleteDir(dir + fname, false))
        return false;
	if (!d.GetNext(&fname))
	  break;
  }
  d.GetFirst(&fname, wxEmptyString, wxDIR_FILES | wxDIR_HIDDEN);
  while(1)
  {
	if (fname.Mid(0,4).Lower() == _T("menu") ||
		fname.Mid(0,5).Lower() == _T("title"))
	  if (!DeleteFile(dir + fname))
	    return false;
	if (!d.GetNext(&fname))
	  break;
  }
  d.Open(wxGetHomeDir());
  wxLogNull log;
  wxRmdir(dir);
  return true;
}

bool ProgressDlg::Exec(wxString command, wxString inputFile, wxString outputFile)
{
  ProcessExecute exec(this);
  return exec.Execute(command, inputFile, outputFile);
}

