/////////////////////////////////////////////////////////////////////////////
// Name:        ProgressDlg.h
// Purpose:     Progress of generation/burning dialog
// Author:      Alex Thuering
// Created:     14.08.2004
// RCS-ID:      $Id: ProgressDlg.h,v 1.8 2005/04/11 16:39:24 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////
 
#ifndef PROGRESS_DLG_H
#define PROGRESS_DLG_H

#include "DVD.h"
#include "BurnDlg.h"

// begin wxGlade: dependencies
// end wxGlade

class ProgressDlg: public wxDialog
{
  public:
    // begin wxGlade: ProgressDlg::ids
    enum {
        HIDE_BT_ID = 1001,
        ICONIZE_BT_ID = 1002
    };
    // end wxGlade
    ProgressDlg(wxWindow* parent);
    void Start(BurnDlg* burnDlg, DVD* dvd);
	
    void AddSummaryMsg(const wxString& message, const wxString& details = wxEmptyString,
	  const wxColour& colour = wxNullColour);
    void AddDetailMsg(const wxString& message, const wxColour& colour = wxNullColour);
	void AddDetailText(const wxString& text);
	  
	void SetSummaryRange(int max) { m_summaryGauge->SetRange(max*100); m_lastSummary = 0; }
	void IncSummaryStep(int value = 1) { m_lastSummary += value*100; m_summaryGauge->SetValue(m_lastSummary); }
	void UpdateSummary();
	
	void SetDetailRange(int max) { m_detailsGauge->SetRange(max); m_detailsGauge->SetValue(0); }
	int GetDetailStep() { return m_detailsGauge->GetValue(); }
	void SetDetailStep(int value) { m_detailsGauge->SetValue(value); UpdateSummary(); }
	void IncDetailStep(int value = 1) { SetDetailStep(GetDetailStep() + value); }
	
	void Failed(const wxString& message);
	void FailedExec(const wxString& command);
    void End();
    bool IsCanceled();

  protected:
    // begin wxGlade: ProgressDlg::methods
    void set_properties();
    void do_layout();
    // end wxGlade
    // begin wxGlade: ProgressDlg::attributes
    wxStaticText* m_summaryLabel;
    wxTextCtrl* m_summaryText;
    wxGauge* m_summaryGauge;
    wxGauge* m_detailsGauge;
    wxStaticText* m_detailsLabel;
    wxTextCtrl* m_detailsText;
    wxButton* m_detailsBt;
    wxButton* m_minimizeBt;
    wxButton* m_cancelBt;
    // end wxGlade
	wxSizer* m_panelSizer;
    
	int m_lastSummary;
    bool m_cancel;
    void OnCancel(wxCommandEvent& WXUNUSED(event));
	wxString m_detailsBtLabel;
	void OnHideDetails(wxCommandEvent& WXUNUSED(event));
	void OnMinimize(wxCommandEvent& WXUNUSED(event));
	
	wxWindowDisabler* m_winDisabler;
	void Run(BurnDlg* burnDlg, DVD* dvd);
	bool GenerateMenu(Menu* menu, const wxString& menuFile);
	bool MakeVob(const wxString& videoFile, const wxString& audioFile,
	  const wxString& vobFile);
	bool FixVob(const wxString& mpegFile, const wxString& vobFile);
    bool DeleteFile(wxString fname);
    bool DeleteDir(wxString dir, bool subdirs = false);
    bool DeleteTempDir(wxString dir);
	bool Exec(wxString command,
	  wxString inputFile = wxEmptyString, wxString outputFile = wxEmptyString);
    
  private:
    DECLARE_EVENT_TABLE()
};

#endif // PROGRESS_DLG_H
