/////////////////////////////////////////////////////////////////////////////
// Name:        TitleThumbnails.h
// Purpose:     Title thumbnails
// Author:      Alex Thuering
// Created:		27.01.2003
// RCS-ID:      $Id: TitleThumbnails.cpp,v 1.32 2005/05/16 13:10:58 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "TitleThumbnails.h"
#include "TitlePropDlg.h"
#include "MenuPropDlg.h"
#include "MPEG.h"
#include "Config.h"
#include <wxVillaLib/ThumbnailFactory.h>
#include <wxVillaLib/ImageProc.h>
#include <wx/splitter.h>
#include <wx/dnd.h>
#include <wx/utils.h>

//////////////////////////////////////////////////////////////////////////////
///////////////////////////// TitleDnDFile ///////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

class TitleDnDFile : public wxFileDropTarget
{
  public:
    TitleDnDFile(TitleThumbnails* pOwner): m_pOwner(pOwner) 
	{
	  SetDataObject(new wxFileDataObject);
#if wxCHECK_VERSION(2,6,0) // wxWidgets bug 1100327
	  SetDefaultAction(wxDragMove);
#endif
	}
	
	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
	{
	  wxFileDropTarget::OnEnter(x, y, def);
	  return def;
	}
	
	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
	{
	  defDrag = def; // wxWidgets bug 1100327
	  wxFileDropTarget::OnDragOver(x, y, def);
	  return def;
	}
	
	virtual wxDragResult OnData(wxCoord x, wxCoord y, wxDragResult def)
	{
#if wxCHECK_VERSION(2,6,0)
	  defDrag = def; // wxWidgets bug 1100327
#endif
	  wxFileDropTarget::OnData(x, y, def);
	  return def;
	}
	
	virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
	{
	  bool res = false;
	  for (int i = 0; i<(int)filenames.Count(); i++)
		res = AddFile(filenames[i]) || res;
	  return res;
	}
	
	bool AddFile(wxString filename)
	{
	  if (wxImage::FindHandler(filename.AfterLast('.').Lower(), -1))
		return false;
	  else if (wxThumbnails::IsVideo(filename))
	  {
		if (!MPEG::IsValid(filename))
		{
		  wxLogError(filename + _(": not valid mpeg-file"));
		  return false;
		}
		m_pOwner->InsertTitle(filename, -1, defDrag == wxDragMove);
		return true;
	  }
	  else if (wxThumbnails::IsAudio(filename))
	  {
		m_pOwner->InsertTitle(filename);
		return true;
	  }
	  wxLogError(filename + _(": unknown format"));
	  return false;
	}
	
  protected:
    TitleThumbnails *m_pOwner;
	wxDragResult defDrag;
};

//////////////////////////////////////////////////////////////////////////////
/////////////////////////// TitleThumbnails //////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

enum
{
  MENU_TITLE_DEL_ID = 2650,
  MENU_TITLE_PROP_ID,
  MENU_TITLE_ADD_ID,
  MENU_TITLE_MOVE_ID,
  MENU_MAIN_ADD_ID,
  MENU_ADD_MENU_ID,
  MENU_ADD_VMMENU_ID,
  MENU_ADD_TITLESET_ID,
  MENU_MOVE_TITLE_LEFT_ID,
  MENU_MOVE_TITLE_RIGHT_ID
};

BEGIN_EVENT_TABLE(TitleThumbnails, wxThumbnails)
  EVT_MENU(MENU_ADD_MENU_ID, TitleThumbnails::OnAddMenu)
  EVT_MENU(MENU_ADD_VMMENU_ID, TitleThumbnails::OnAddVmMenu)
  EVT_MENU(MENU_ADD_TITLESET_ID, TitleThumbnails::OnAddTitleset)
  EVT_UPDATE_UI(MENU_ADD_TITLESET_ID, TitleThumbnails::OnAddTitlesetUpdateUI)
  EVT_MENU(MENU_MOVE_TITLE_LEFT_ID, TitleThumbnails::OnMoveTitleLeft)
  EVT_MENU(MENU_MOVE_TITLE_RIGHT_ID, TitleThumbnails::OnMoveTitleRight)
  EVT_UPDATE_UI(MENU_MOVE_TITLE_LEFT_ID, TitleThumbnails::OnMoveTitleLeftUpdateUI)
  EVT_UPDATE_UI(MENU_MOVE_TITLE_RIGHT_ID, TitleThumbnails::OnMoveTitleRightUpdateUI)
  EVT_MENU(MENU_TITLE_DEL_ID, TitleThumbnails::OnDelete)
  EVT_MENU(MENU_TITLE_PROP_ID, TitleThumbnails::OnProps)
END_EVENT_TABLE()

DEFINE_EVENT_TYPE(EVT_COMMAND_DVD_CHANGED)

TitleThumbnails::TitleThumbnails(wxWindow* parent, int id):
  wxThumbnails(parent, id)
{
  SetThumbSize(112, 76);
  SetOrientaion(wxTHUMB_HORIZONTAL);
  SetDropTarget(new TitleDnDFile(this));
  EnableDragging(false);
  m_dvd = NULL;
  m_selectedMenu = -1;
  
  wxMenu* addMenu = new wxMenu;
  addMenu->Append(MENU_ADD_MENU_ID, _("&Menu"));
  addMenu->Append(MENU_ADD_VMMENU_ID, _("&vmMenu"));
  addMenu->Append(MENU_ADD_TITLESET_ID, _("&Titleset"));
  
  wxMenu* moveMenu = new wxMenu;
  moveMenu->Append(MENU_MOVE_TITLE_LEFT_ID, _("&Left"));
  moveMenu->Append(MENU_MOVE_TITLE_RIGHT_ID, _("&Right"));
  
  m_mainMenu = new wxMenu;
  m_mainMenu->Append(MENU_MAIN_ADD_ID, _("&Add"), addMenu);
  
  m_titleMenu = new wxMenu;
  m_titleMenu->Append(MENU_TITLE_ADD_ID, _("&Add"), addMenu);
  m_titleMenu->Append(MENU_TITLE_MOVE_ID, _("&Move"), moveMenu);
  m_titleMenu->Append(MENU_TITLE_DEL_ID, _("&Delete"));
  m_titleMenu->Append(MENU_TITLE_PROP_ID, _("&Properties..."));
  
  SetPopupMenu(m_titleMenu);
  SetGlobalPopupMenu(m_mainMenu);
}

TitleThumbnails::~TitleThumbnails()
{
}

void TitleThumbnails::SetDVD(DVD* dvd)
{
  m_dvd = dvd;
  SetSelected(-1);
  m_selectedMenu = 0;
  UpdateShow();
}

void TitleThumbnails::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index)
{
  if (!m_dvd)
	return;
  int id = thumb.GetId();
  int tsi = DVD::GetTsi(id);
  int pgci = DVD::GetPgci(id);
  int vobi = DVD::GetVobi(id);
  bool isMenu = DVD::IsMenu(id);
  
  wxMemoryDC dc;
  dc.SelectObject(bmp);
  dc.BeginDrawing();  
  int x = m_tBorder/2;
  int y = m_tBorder/2;
  int tw = bmp.GetWidth() - m_tBorder + 2;
  int th = bmp.GetHeight() - m_tBorder + 2;
  int titleHeight = m_tTextHeight + 2;
  // background
  dc.SetPen(wxPen(*wxBLACK,0,wxTRANSPARENT));
  dc.SetBrush(wxBrush(GetBackgroundColour(), wxSOLID));
  dc.DrawRectangle(0, 0, bmp.GetWidth(), bmp.GetHeight());
  // rounded rectangle
  wxColour rectColour = isMenu ? wxColour(64,64,64) : wxColour(0,0,0);
  dc.SetBrush(wxBrush(rectColour, wxSOLID));
  dc.DrawRoundedRectangle(x, y, tw, th, 20);
  // draw begin & end of titleset
  if (IsBeginOfTitleset(tsi, pgci, vobi, isMenu))
  {
	int h = bmp.GetHeight();
	dc.SetPen(wxPen(*wxRED,0,wxSOLID));
	dc.DrawLine(1, 1, 10, 1);
	dc.DrawLine(1, 1, 1, 10);
	dc.DrawLine(1, h-2, 10, h-2);
	dc.DrawLine(1, h-2, 1, h-11);
  }
  if (IsEndOfTitleset(tsi, pgci, vobi, isMenu))
  {
	int h = bmp.GetHeight();
	int w = bmp.GetWidth();
	dc.SetPen(wxPen(*wxRED,0,wxSOLID));
	dc.DrawLine(w-1, 1, w-10, 1);
	dc.DrawLine(w-1, 1, w-1, 10);
	dc.DrawLine(w-1, h-2, w-10, h-2);
	dc.DrawLine(w-1, h-2, w-1, h-11);
  }
  // image
  wxBitmap img;
  if (isMenu)
  {
	if (!thumb.GetBitmap().Ok())
	{
	  Menu* menu = m_dvd->GetMenu(tsi, pgci);
	  if (menu)
	    thumb.SetBitmap(menu->GetImage(mdALL, m_tWidth, m_tHeight-titleHeight));
	}
	img = thumb.GetBitmap();
  }
  else
	img = thumb.GetBitmap(this, m_tWidth, m_tHeight-titleHeight);
  if (index == m_pointed)
  {
    wxImage imgTmp = img.ConvertToImage();
    wxAdjustBrightness(imgTmp, wxRect(0,0,img.GetWidth(),img.GetHeight()),
      POINTED_BRIGHTNESS);
    img = wxBitmap(imgTmp);
  }
  wxRect imgRect(x + (m_tWidth-img.GetWidth())/2,
	y + titleHeight + (m_tHeight-titleHeight-img.GetHeight())/2,
    img.GetWidth(), img.GetHeight());      
  dc.DrawBitmap(img, imgRect.x, imgRect.y);
  // outline
  if (index == m_selectedMenu)
  {
	dc.SetPen(wxPen(*wxBLUE,0,wxSOLID));
	dc.SetBrush(wxBrush(*wxBLACK, wxTRANSPARENT));
	dc.DrawRectangle(imgRect.x-1, imgRect.y-1, imgRect.width+1, imgRect.height+1);
  }
  // draw title caption
  dc.SetTextForeground(*wxWHITE);
  wxString caption;
  if (isMenu && tsi<0)
	caption = _("vmMenu");
  else if (isMenu)
	caption = _("Menu");
  else
	caption = _("Title");
  caption += wxString::Format(_T(" %d"), pgci+1);
  if (vobi>0)
	caption += wxString::Format(_T("-%d"), vobi+1);
  int sw, sh;
  dc.GetTextExtent(caption, &sw, &sh);
  int tx = x + (m_tWidth-sw)/2;
  int ty = y + (titleHeight-sh)/2;
  dc.DrawText(caption, tx, ty);
  // draw caption
  for (int i=0; i<thumb.GetCaptionLinesCount(m_tWidth-m_tCaptionBorder); i++)
  {
	wxString caption = thumb.GetCaption(i);
	dc.GetTextExtent(caption, &sw, &sh);
	tx = x + (m_tWidth-sw)/2;
	ty = y + m_tHeight + i*m_tTextHeight + (m_tTextHeight-sh)/2;
	dc.DrawText(caption, tx, ty);
  }
  
  dc.EndDrawing();
  dc.SelectObject(wxNullBitmap);
}

int TitleThumbnails::GetCaptionHeight(int begRow, int count)
{
  int capHeight = 0;
  for (int i=begRow; i<begRow+count; i++)
	if (i < (int) m_tCaptionHeight.GetCount())
	capHeight += m_tCaptionHeight[i]>0? m_tCaptionHeight[i] : 1;
  return capHeight*m_tTextHeight; 
}

void TitleThumbnails::UpdateShow()
{
  wxThumbnailFactory::ClearQueue(this);
  
  int selectedMenuId = -1; // to update m_selectedMenu
  if (m_selectedMenu>=0 && m_selectedMenu<(int)m_items.Count())
	selectedMenuId = m_items[m_selectedMenu]->GetId();
  WX_CLEAR_ARRAY(m_items)
  Menus* menus = &m_dvd->GetVmgm();
  for (int pgci = 0; pgci<(int)menus->Count(); pgci++)
  {
	Pgc* pgc = (*menus)[pgci];
	if (!pgc->GetVobs().Count())
	  continue;
	wxThumb* thumb = new wxThumb(_T(""));
	thumb->SetId(DVD::MakeId(-1, pgci, 0, true));
	m_items.Add(thumb);
  }
  for (int tsi = 0; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++)
  {
	Titleset* ts = m_dvd->GetTitlesets()[tsi];
	menus = &ts->GetMenus();
	for (int pgci = 0; pgci<(int)menus->Count(); pgci++)
	{
	  Pgc* pgc = (*menus)[pgci];
	  if (!pgc->GetVobs().Count())
		continue;
	  wxThumb* thumb = new wxThumb(_T(""));
	  thumb->SetId(DVD::MakeId(tsi, pgci, 0, true));
	  m_items.Add(thumb);
	}
	for (int pgci = 0; pgci<(int)ts->GetTitles().Count(); pgci++)
	{
	  Pgc* pgc = ts->GetTitles()[pgci];
	  for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++)
	  {
		Vob* vob = pgc->GetVobs()[vobi];
		wxString fname = vob->GetFilename();
		if (!fname.length())
		  fname = vob->GetVideoFilename();
		if (!fname.length())
		  fname = vob->GetAudioFilename();
		wxThumb* thumb = new wxThumb(fname,
		  fname.AfterLast(wxFILE_SEP_PATH).BeforeLast('.'));
		thumb->SetId(DVD::MakeId(tsi, pgci, vobi, false));
		m_items.Add(thumb);
	  }
	}
  }
  if (selectedMenuId>=0) // update m_selectedMenu
	m_selectedMenu = GetIndex(selectedMenuId);
  UpdateProp();
  Refresh();
  ((wxSplitterWindow*)GetParent())->SetSashPosition(
	((wxSplitterWindow*)GetParent())->GetSashPosition()+(m_rows>1?0:1000));
  SendDvdChangedEvent();
}

int TitleThumbnails::GetIndex(int tsi, int pgci, int vobi, bool menu)
{
  return GetIndex(DVD::MakeId(tsi, pgci, vobi, menu));
}

int TitleThumbnails::GetIndex(int id)
{
  for (int i=0; i<(int)m_items.Count(); i++)
  {
	//wxMessageBox(wxString::Format(_T("find %d ? %d"), id, m_items[i]->GetId()));
	if (m_items[i]->GetId() == id)
	  return i;
  }
  return -1;
}

void TitleThumbnails::InsertTitle(const wxString& fname, int pos, bool createTitle)
{
  int tsi = m_dvd->GetTitlesets().Count()-1;
  PgcArray& pgcs = m_dvd->GetPgcArray(tsi, false);
  int pgci = pgcs.Count()-1;
  Pgc* pgc = pgci>=0 ? pgcs[pgci] : NULL;
  int vobi = pgci>=0 ? (int)pgc->GetVobs().Count()-1 : -1;
  Vob* vob = pgci>=0 && vobi>=0 ? pgc->GetVobs()[vobi] : NULL;
  bool lastTitleComplete = pgci<0 || vobi<0 ||
	vob->GetFilename().length() ||
	(vob->GetVideoFilename().length() && vob->GetAudioFilename().length());
  if (lastTitleComplete)
  {
	Vob* vob = new Vob(_T(""));
	if (wxThumbnails::IsAudioVideo(fname))
	  vob->SetFilename(fname);
	else if (wxThumbnails::IsVideo(fname))
	  vob->SetVideoFilename(fname);
	else if (wxThumbnails::IsAudio(fname))
	  vob->SetAudioFilename(fname);
	vob->SetChapters(_T("15:00,30:00,45:00,1:00:00"));
	if (createTitle || pgc == NULL)
	{
	  pgc = new Pgc;
	  pgc->SetPostCommands(_T("call vmgm menu 1;"));
	  pgc->GetVobs().Add(vob);
	  pgcs.Add(pgc);
	}
	else
	  pgc->GetVobs().Add(vob);
  }
  else
  {
	if (wxThumbnails::IsAudioVideo(fname) ||
	  (wxThumbnails::IsVideo(fname) && vob->GetVideoFilename().length()) ||
	  (wxThumbnails::IsAudio(fname) && vob->GetAudioFilename().length()))
	{
	  wxString msg = _("The last title isn't complete.\n");
	  if (vob->GetVideoFilename().length())
		msg += _("Please add a audio track to it.");
	  else
		msg += _("Please add a video track to it.");
	  wxLogError(msg);
	  return;
	}
	else if (wxThumbnails::IsVideo(fname))
	  vob->SetVideoFilename(fname);
	else if (wxThumbnails::IsAudio(fname))
	  vob->SetAudioFilename(fname);
  }
  UpdateShow();
}

void TitleThumbnails::SelectMenu(int tsi, int pgci)
{
  m_selectedMenu = GetIndex(tsi, pgci, 0, true);
  SendDvdChangedEvent(true);
}

void TitleThumbnails::SelectFirstMenu()
{
  m_selectedMenu = -1;
  for (int tsi=-1; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++)
  {
	if (m_dvd->GetMenu(tsi, 0))
	{
	  m_selectedMenu = GetIndex(tsi, 0, 0, true);
	  break;
	}
  }
  SendDvdChangedEvent(true);
}

bool TitleThumbnails::IsBeginOfTitleset(int tsi, int pgci, int vobi, bool isMenu)
{
  if (tsi<0 || (m_dvd->GetTitlesets().Count()<=1 && m_dvd->GetVmgm().Count()==0))
	return false;
  return pgci == 0 && vobi == 0 &&
	(isMenu || m_dvd->GetTitlesets()[tsi]->GetMenus().Count() == 0);
}

bool TitleThumbnails::IsEndOfTitleset(int tsi, int pgci, int vobi, bool isMenu)
{
  if (tsi<0 || m_dvd->GetTitlesets().Count() < 2 ||
	  tsi == (int)m_dvd->GetTitlesets().Count()-1)
	return false;
  Titleset* ts = m_dvd->GetTitlesets()[tsi];
  return (isMenu &&
		  ts->GetTitles().Count() == 0 &&
		  pgci == (int)ts->GetMenus().Count()-1 &&
		  vobi == (int)ts->GetMenus()[pgci]->GetVobs().Count()-1) ||
		 (!isMenu &&
		  pgci == (int)ts->GetTitles().Count()-1 &&
		  vobi == (int)ts->GetTitles()[pgci]->GetVobs().Count()-1);
}

void TitleThumbnails::SendDvdChangedEvent(bool selectMenu)
{
  wxCommandEvent evt(EVT_COMMAND_DVD_CHANGED, this->GetId());
  evt.SetInt((int)selectMenu);
  GetEventHandler()->ProcessEvent(evt);
}

void TitleThumbnails::OnMouseDown(wxMouseEvent& event)
{
  wxThumbnails::OnMouseDown(event);
  // Popup menu
  if (event.RightDown())
	return;
  // select menu
  if (GetSelected()<0)
	return;
  int id = GetSelectedId();
  if (DVD::IsMenu(id))
	SelectMenu(DVD::GetTsi(id), DVD::GetPgci(id));
}

void TitleThumbnails::OnMouseDClick(wxMouseEvent& WXUNUSED(event))
{
  wxCommandEvent evt;
  OnProps(evt);
}

void TitleThumbnails::AddMenu()
{
  int tsi = m_dvd->GetTitlesets().Count()-1;
  if (GetSelected()>=0)
	tsi = DVD::GetTsi(GetSelectedId());
  int pgci = m_dvd->AddMenu((VideoFormat)s_config.GetDefMenuFormat(), tsi);
  UpdateShow();
  SelectMenu(tsi, pgci);
}

void TitleThumbnails::AddVmMenu()
{
  int tsi = -1;
  int pgci = m_dvd->AddMenu((VideoFormat)s_config.GetDefMenuFormat(), tsi);
  UpdateShow();
  SelectMenu(tsi, pgci);
}

void TitleThumbnails::AddTitleset()
{
  m_dvd->AddTitleset();
  AddMenu();
}

void TitleThumbnails::OnAddTitlesetUpdateUI(wxUpdateUIEvent& event)
{
  int cnt = m_dvd->GetTitlesets().Count();
  event.Enable(cnt>0 &&	m_dvd->GetTitlesets()[cnt-1]->GetTitles().Count());
}

void TitleThumbnails::OnMoveTitleLeft(wxCommandEvent& WXUNUSED(event))
{
  if (GetSelected()<0)
	return;
  int id = GetSelectedId();
  int tsi = DVD::GetTsi(id);
  int pgci = DVD::GetPgci(id);
  bool isMenu = DVD::IsMenu(id);
  PgcArray& pgcs = m_dvd->GetPgcArray(tsi,isMenu);
  if (pgci == 0)
	return;
  Pgc* pgc = pgcs[pgci]; pgcs[pgci] = pgcs[pgci-1]; pgcs[pgci-1] = pgc;
  if (isMenu)
	SelectMenu(tsi, pgci-1);
  UpdateShow();
}

void TitleThumbnails::OnMoveTitleRight(wxCommandEvent& WXUNUSED(event))
{
  if (GetSelected()<0)
	return;
  int id = GetSelectedId();
  int tsi = DVD::GetTsi(id);
  int pgci = DVD::GetPgci(id);
  bool isMenu = DVD::IsMenu(id);
  PgcArray& pgcs = m_dvd->GetPgcArray(tsi,isMenu);
  if (pgci == (int)pgcs.Count()-1)
	return;
  Pgc* pgc = pgcs[pgci]; pgcs[pgci] = pgcs[pgci+1]; pgcs[pgci+1] = pgc;
  if (isMenu)
	SelectMenu(tsi, pgci+1);
  UpdateShow();
}

void TitleThumbnails::OnMoveTitleLeftUpdateUI(wxUpdateUIEvent& event)
{
  if (GetSelected()<0)
	return;
  int id = GetSelectedId();
  int pgci = DVD::GetPgci(id);
  event.Enable(pgci>0);
}

void TitleThumbnails::OnMoveTitleRightUpdateUI(wxUpdateUIEvent& event)
{
  if (GetSelected()<0)
	return;
  int id = GetSelectedId();
  int tsi = DVD::GetTsi(id);
  int pgci = DVD::GetPgci(id);
  bool isMenu = DVD::IsMenu(id);
  PgcArray& pgcs = m_dvd->GetPgcArray(tsi,isMenu);
  event.Enable(pgci<(int)pgcs.Count()-1);
}

void TitleThumbnails::OnDelete(wxCommandEvent& WXUNUSED(event))
{
  if (GetSelected()<0)
	return;
  int id = GetSelectedId();
  int tsi = DVD::GetTsi(id);
  int pgci = DVD::GetPgci(id);
  bool isMenu = DVD::IsMenu(id);
  int vobi = DVD::GetVobi(id);
  if (isMenu)
  {
	Titleset* ts = NULL;
	if (tsi>=0)
	  ts = m_dvd->GetTitlesets()[tsi];
	Menus* menus = &m_dvd->GetVmgm();
	if (ts)
	  menus = &ts->GetMenus();
	if (pgci>=(int)menus->GetCount())
	  return;
	Pgc* pgc = (*menus)[pgci];
	if (vobi>=(int)pgc->GetVobs().Count())
	  return;
	delete pgc->GetVobs()[vobi];
	pgc->GetVobs().RemoveAt(vobi);
	// remove pgc if need
	if (pgc->GetVobs().Count() == 0)
	{
	  delete pgc;
	  menus->RemoveAt(pgci);
	}
	// remove titleset if need
	if (ts &&
		ts->GetTitles().Count() == 0 &&
		ts->GetMenus().Count() == 0 &&
		m_dvd->GetTitlesets().Count() > 1)
	{
	  delete ts;
	  m_dvd->GetTitlesets().RemoveAt(tsi);
	}
	// select next menu
	if (GetSelected() == m_selectedMenu)
	{
		if (pgci >= (int)menus->Count())
		  pgci = menus->Count()-1;
		if (pgci>=0)
		  SelectMenu(tsi, pgci);
		else
		  SelectFirstMenu();
	}
	else if (m_selectedMenu>=0)
	{
	  wxThumb* mThumb = m_items[m_selectedMenu];
	  int mTsi = (mThumb->GetId()>>24)-1;
	  int mPgci = ((mThumb->GetId()>>8) & 0xFFFF) / 2;
	  if (mTsi == tsi && mPgci > pgci)
		m_selectedMenu--;
	}
  }
  else // title
  {
	if (tsi>=(int)m_dvd->GetTitlesets().Count())
	  return;
	Titleset* ts = m_dvd->GetTitlesets()[tsi];
	
	//title
	if (pgci>=(int)ts->GetTitles().GetCount())
	  return;
	Pgc* pgc = ts->GetTitles()[pgci];
	if (vobi>=(int)pgc->GetVobs().Count())
	  return;
	// remove vob
	delete pgc->GetVobs()[vobi];
	pgc->GetVobs().RemoveAt(vobi);
	// remove title if need
	if (pgc->GetVobs().Count() == 0)
	{
	  delete pgc;
	  ts->GetTitles().RemoveAt(pgci);
	}
	// remove titleset if need
	if (ts->GetTitles().Count() == 0 &&
		ts->GetMenus().Count() == 0 &&
		m_dvd->GetTitlesets().Count() > 1)
	{
	  delete ts;
	  m_dvd->GetTitlesets().RemoveAt(tsi);
	}
  }
  SetSelected(-1);
  UpdateShow();
}

void TitleThumbnails::OnProps(wxCommandEvent& WXUNUSED(event))
{
  if (GetSelected()<0)
	return;
  int id = GetSelectedId();
  int tsi = DVD::GetTsi(id);
  int pgci = DVD::GetPgci(id);
  bool isMenu = DVD::IsMenu(id);
  int vobi = DVD::GetVobi(id);
  if (isMenu)
  {
	MenuPropDlg propDlg(this, m_dvd, tsi, pgci);
	if (propDlg.ShowModal() == wxID_OK)
	  SendDvdChangedEvent(true);
  }
  else //title
  {
	PgcArray& pgcs = m_dvd->GetPgcArray(tsi, false);
	Pgc* pgc = pgcs[pgci];
	TitlePropDlg propDlg(this, m_dvd, &pgcs.GetAudio(), &pgcs.GetVideo(),
	  pgc, pgc->GetVobs()[vobi]);
	if (propDlg.ShowModal() == wxID_OK)
	  SendDvdChangedEvent();
  }
}
